#!/usr/bin/env python
# -*- coding:utf-8 -*-
import re, os, logging, traceback
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.image import MIMEImage
from email.header import Header
from email.utils import formataddr


class SmtpServer(object):
    def __init__(self, **kwargs):
        debuglevel = kwargs.pop("debuglevel", False)
        smtp_ssl = kwargs.get("smtp_ssl", True)
        keyfile = kwargs.pop("keyfile", None)
        certfile = kwargs.pop("keyfile", None)
        source_address = kwargs.pop("source_address", None)
        timeout = kwargs.pop("timeout", None)
        local_hostname = kwargs.pop("local_hostname", None)
        context = kwargs.pop("context", None)

        self.smtp_host = kwargs["smtp_host"]
        self.smtp_post = kwargs["smtp_post"]  # SMTP端口25, SMTP_SSL安全连接端口号465或587
        self.smtp_ssl = kwargs.get("smtp_ssl", True)  # 是否使用ssl
        self.username = kwargs.get("username", "")  # 登录SMTP服务器的账号
        self.password = kwargs.get("password", "")  # SMTP客户端授权密码
        self.smtp_email = kwargs.get("smtp_email", "")  # "发件人的昵称<xxxxx@qq.com>"

        if smtp_ssl:
            connection_class = smtplib.SMTP_SSL(self.smtp_host, self.smtp_post, local_hostname, keyfile, certfile,
                                                timeout, source_address, context)
        else:
            connection_class = smtplib.SMTP(self.smtp_host, self.smtp_post, local_hostname, timeout, source_address)

        self.smtp = connection_class
        if debuglevel:
            self.smtp.set_debuglevel(True)  # debug模式打印详情

    def connection(self):
        self.smtp.connect(self.smtp_host, self.smtp_post)
        self.smtp.login(self.username, self.password)

    def send(self, to_emails: [str, list, tuple],
             title, message="", html_message=None,
             images=None, attachs=None, from_email=None):
        """
        from_email: yourself email
        to_mails:"one@one.org" or ["one@one.org","two@two.org","three@three.org","four@four.org"]
        message: 文本内容
        html_message: html内容，注意邮件的 HTML 文本中一般邮件服务商添加外链是无效的
                <p>邮件文本内容</p>
                <p><a href="http://www.xxx.com">链接</a></p>
                <p>内容中的图片未被显示时，需添加发件人信任。或者点击邮件页面中的"显示图片 信任此发件人的图片"</p>
                <p><img src="http://www.url.com/x.png"></p>
                <p>图片标签：以下需添加image1的标签图片</p>
                <p><img src="cid:image1"></p>

        images: 需要在html_message中出现的图片，
        [path1, path2] or [image_buffer1, image_buffer2]，
        用免费的腾讯邮箱发送，img标签不能正常显示

        推荐在html_message使用完整的图片地址实现

        attachs: 附件地址或者文件数据列表
        [path1, path2] or [buffer1, buffer2]
        """
        mime_message = MIMEMultipart("related")
        # mime_message = MIMEText(message, 'plain', 'utf-8')

        # mime_message['From'] = Header(from_email, 'utf-8')  # 发送者
        # mime_message['To'] = Header(",".join(to_emails), 'utf-8')  # 接收者
        # mime_message['To'] = formataddr(("指定收件人昵称", ",".join(to_emails)))

        from_email = from_email or self.smtp_email
        # 标题
        mime_message['Subject'] = Header(title, 'utf-8')
        # 发件人
        if re.search(r'<.+>', from_email):
            smtp_name, from_email = from_email.split("<")
        else:
            smtp_name, from_email = from_email, from_email
        mime_message['From'] = formataddr((smtp_name.rstrip(" "), from_email.rstrip(">")))
        # 收件人
        to_emails = [to_emails] if not isinstance(to_emails, (list, tuple)) else to_emails
        for email in to_emails:
            mime_message.add_header("to", email)  # 接收者

        # 文本
        if message:
            mime_message.attach(MIMEText(message, 'plain', 'utf-8'))

        # html
        if html_message:
            mime_html = MIMEMultipart("alternative")
            mime_message.attach(mime_html)
            mime_html.attach(MIMEText(html_message, 'html', 'utf-8'))

            # 图片: 需提供html_message内容,并预留image为前缀的名称标签
            if images:
                images = [images] if not isinstance(images, (list, tuple)) else images
                for i, image in enumerate(images):
                    if os.path.isfile(image):
                        with open(image, 'rb') as p:
                            data = p.read()
                    else:
                        data = image
                    mime_image = MIMEImage(data)
                    # 定义图片 ID，在 HTML 文本中引用
                    mime_image.add_header('Content-ID', 'image%s' % (i + 1))
                    mime_message.attach(mime_image)
        # 附件
        if attachs:
            attachs = [attachs] if not isinstance(attachs, (list, tuple)) else attachs
            for attach in attachs:
                if os.path.isfile(attach):
                    with open(attach, 'rb') as p:
                        data = p.read()
                else:
                    data = attach
                mime_attach = MIMEText(data, 'base64', 'utf-8')
                mime_attach["Content-Type"] = 'application/octet-stream'
                mime_attach["Content-Disposition"] = 'attachment; filename="%s"' % os.path.basename(attach)
                mime_message.attach(mime_attach)

        self.connection()
        result = self.smtp.sendmail(from_email, to_emails, mime_message.as_string())
        self.smtp.quit()
        return result

    def send_email(self, to_emails: [str, list, tuple],
                   title, message="", html_message=None,
                   images=None, attachs=None, from_email=None):
        try:
            result = self.send(to_emails,
                               title, message, html_message,
                               images, attachs, from_email)
        except Exception as e:
            logging.error(traceback.format_exc())
            result = False
        else:
            to_emails = [to_emails] if not isinstance(to_emails, (list, tuple)) else to_emails
            logging.debug("邮件发送成功:%s" % ",".join(to_emails))
            result = result or True
        finally:
            return result


if __name__ == "__main__":
    # SMTP服务器地址
    smtp_host = "smtp.qq.com"
    # SMTP服务端口
    smtp_post = 465  # SMTP端口25, SMTP_SSL安全连接端口号465或587
    # 是否使用ssl
    smtp_ssl = True
    # 登录SMTP服务器的账号
    username = "123220663@qq.com"
    # SMTP客户端授权密码
    password = "ujzrzohozvcybggd"
    # 发件人邮箱
    smtp_email = "发件人的昵称<xxxxx@qq.com>"

    cli = SmtpServer(smtp_host=smtp_host,
                     smtp_post=smtp_post,
                     smtp_ssl=smtp_ssl,
                     debuglevel=False,
                     smtp_email=smtp_email,
                     username=username,
                     password=password
                     )
    cli.send_email(
        # ["取名字不难<452809783@qq.com>", "昏昏暗暗<123220663@qq.com>"],
        ["452809783@qq.com", "123220663@qq.com"],
        "邮件标题6",
        message="文本",
        html_message="""
        <p style="color:red">邮件文本内容</p>
        <p><a href="http://www.xxx.com">链接</a></p>
        <p>内容中的图片未被显示时，需添加发件人信任。或者点击邮件页面中的"显示图片 信任此发件人的图片"</p>
        <p><img src="https://www.runoob.com/wp-content/uploads/2016/04/smtp4.jpg"></p>
        <p>图片标签：以下需添加image1的标签图片</p>
        <p><img src="cid:image1"></p>
        <p>2222</p>
        """,
        images=["/Users/apple/Downloads/WX20220714-160954@2x.png"],
        attachs=["/Users/apple/Downloads/WX20220714-154222@2x.png"]
    )



# coding=utf-8
import sys
from base import BaseModel
from lcyframe import yaml2py
from lcyframe import ServicesFindet
from lcyframe.beanstalk_server import BeanstalkWorker
from context import InitContext

# 监听的tube的优先级：命令行参数>配置文件指定>消费者指定的所有tube（默认）
# python beanstalk_start.py --watch=tube1,tube2,tube3
watch = sys.argv[1].replace(" ", "").strip("--watch=").strip("-W").lstrip("=").split(",") if len(sys.argv) > 1 else []
sys.argv = sys.argv[:1]
config = InitContext.get_context()
if watch:
    config["beanstalk_config"]["watch"] = watch

tasks = BeanstalkWorker(**config)
yaml2py.impmodule(BaseModel, model_dir="model")
ServicesFindet(tasks, BaseModel)(config)
tasks.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

handler_j2 = r''' #!/usr/bin/env python
# -*- coding:utf-8 -*-
from lcyframe.libs.route import route
from lcyframe.base import BaseHandler, BaseModel as Model
{% for resource in resources %}
@route("{{resource.api}}")
class {{resource.handler_name}}(BaseHandler):
    """
    {{resource.description}}
    """
{% for method, m in resource.method.items() %}
    def {{method}}(self):
        """{{m.summary}}
        {{m.description}}
        {% if m.parameters %}
        Request extra query params:
        {% for p in m.parameters -%}
           - {{p.name}} # {{p.description}} type: {{p.type}}
        {% endfor -%}
        {% endif %}

        :return:
        :rtype:
        """
        pass
{% endfor %}
{% endfor -%}'''


model_j2 = r''' #!/usr/bin/env python
# -*- coding:utf-8 -*-
from lcyframe.base import BaseModel
from bson.objectid import ObjectId
{% for resource in resources[:1] %}
class {{resource.schame_name}}(object):
    """
    {{resource.name}} 表
    """
    collection = "{{resource.name}}"
    structure = {
        # ID
        '_id': str(ObjectId()),
    }
class {{resource.model_name}}(BaseModel, {{resource.schame_name}}):

    @classmethod
    def get(cls, *args, **kwargs):
        """
        单条记录
        :return:
        :rtype:
        """
        pass

    @classmethod
    def query(cls, *args, **kwargs):
        """
        列表
        :return:
        :rtype:
        """
        pass

    @classmethod
    def create(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete(cls, *args, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        pass

{% endfor -%}'''


import logging
from datetime import datetime

from contextlib import suppress
from django.conf import settings
from django.urls import resolve
from django.db.models import Q
from mongoengine.queryset.visitor import Q as MONGO_Q

from article.models import Article, ArticleFile
from rbac.publick_tools import get_ur_subdomain
from system_setting.models import Setting
from vulnerability.models import Vulnerability

logger = logging.getLogger("django")


def check_permission_hook(request):
    print("check_permission_hook")
    return True


# 函数名称最好与权限codename保持一致，后续可用于自动导入权限条目标
def service_center_article_get(request):
    """
    根据审核角色获取资讯列表
    ------如果当前用户是审核角色，获取全部待审核|全部已发布|当前用户的草稿|当前用户的回退
    ------如果登当前用户是非审核角色， 获取全部已发布|当前用户的草稿|当前用户的回退
    :param request:
    :return:
    """
    msg = "ok"
    group_name_list = list(request.user.groups.all().values_list("name", flat=True))
    audict_role = set(group_name_list) & set(Setting.objects.get(key="ARTICLE_AUDIT_ROLE").value)
    q = Q()
    q.connector = "OR"
    q.children.append(("publisher_id", request.user.id))
    q.children.append(("news_state", Article.NEWS_ON_PUBLISH))
    if audict_role:
        q.children.append(("news_state", Article.NEWS_WAIT_AUDIT))
    permission_queryset = Article.objects.filter(q).extra(
        select={
            "content": "left(article.content, 80)",
            "publisher": "select username from user where article.publisher_id = user.id",
            "auditer": "select username from user where article.auditer_id = user.id",
            "cover_name_url": 'select concat("{\'name\':\'", article_file.name, "\',\'url\':\'", article_file.file, "\'}") as cover_name_url from article_file '
                              'where article_file.type = %d and article_file.article_id=article.id' % ArticleFile.COVER,
        }
    )
    setattr(request, "permission_queryset", permission_queryset)
    return True, msg


def service_center_article_pk_delete(request):
    """
    删除资讯
    ------如果该资讯是当前用户发布的，则允许删除，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and qs.first().publisher_id != request.user.id:
        return False, msg
    return True, msg


def service_center_article_pk_agree_put(request):
    """
    同意发布资讯
    ------如果当前用户角色属于审核角色，则允许同意发布该资讯，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    group_name_list = list(request.user.groups.all().values_list("name", flat=True))
    audit_role = set(group_name_list) & set(Setting.objects.get(key="ARTICLE_AUDIT_ROLE").value)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and not audit_role:
        return False, msg
    return True, msg


def service_center_article_pk_disagree_put(request):
    """
    不同意发布资讯
    ------如果当前用户角色属于审核角色，则允许不同意发布该资讯，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    group_name_list = list(request.user.groups.all().values_list("name", flat=True))
    audit_role = set(group_name_list) & set(Setting.objects.get(key="ARTICLE_AUDIT_ROLE").value)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and not audit_role:
        return False, msg
    return True, msg


def service_center_article_pk_submit_put(request):
    """
    编辑资讯(提交)
    ------如果该资讯是当前用户发布的，则允许提交，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and qs.first().publisher_id != request.user.id:
        return False, msg
    return True, msg


def service_center_article_pk_save_put(request):
    """
    编辑资讯(保存)
    ------如果该资讯是当前用户发布的，则允许保存，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and qs.first().publisher_id != request.user.id:
        return False, msg
    return True, msg


def service_center_article_file_pk_delete(request):
    """
    删除资讯附件
    ------如果该资讯是当前用户发布的，则允许保存，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    qs = ArticleFile.objects.filter(id=kwargs.get("pk")).extra(
        select={"publisher_id": "article.publisher_id"},
        tables=["article"],
        where=["article.id = article_file.article_id"],
    )
    if qs.exists() and qs.first().publisher_id != request.user.id:
        return False, msg
    return True, msg


def service_center_article_delete_delete(request):
    """
    批量删除资讯
    ------如果选中的所有资讯是当前用户发布的，则允许删除，否则都不允许删除
    :param request:
    :return:
    """
    msg = "ok"
    ids = []
    for i in request.GET.get("ids").split(","):
        with suppress(ValueError, TypeError):
            ids.append(int(i))
    publisher_id_list = list(set(Article.objects.filter(Q(id__in=ids)).values_list("publisher_id", flat=True)))
    if len(publisher_id_list) == 1 and publisher_id_list[0] == request.user.id:
        return True, msg
    return False, msg


def service_center_article_pk_istop_put(request):
    """
    置顶开关
    ------如果当前用户角色属于审核角色，则允许置顶该资讯，否则不允许
    :param request:
    :return:
    """
    msg = "ok"
    func, args, kwargs = resolve(request.path)
    group_name_list = list(request.user.groups.all().values_list("name", flat=True))
    audit_role = set(group_name_list) & set(Setting.objects.get(key="ARTICLE_AUDIT_ROLE").value)
    qs = Article.objects.filter(id=kwargs.get("pk"))
    if qs.exists() and not audit_role:
        return False, msg
    return True, msg


# def tasks_tasks_post(request):
#     """
#     扫描任务判断
#     ------判断user的公司的权限组
#     :param request:
#     :return:
#     """
#     msg = "ok"
#
#     try:
#         if Tasks.objects.filter(
#                 company_id=request.user.company_id,
#                 create_time__year=datetime.now().year,
#                 create_time__month=datetime.now().month
#         ).exclude(state=Tasks.STATE_ABEND).count() >= request.user.company.groups.role.scan_times:
#             msg = "扫描任务次数达到上限，请联系管理员"
#             return False, msg
#     except Exception as e:
#         msg = "用户异常，请联系客服"
#         return False, msg
#
#     return True, msg


def user_company_user_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    # role_list = request.user.groups.all().values_list("id", flat=True)
    # if settings.ADMIN_ROLE_GROUP not in role_list:
    #     q = Q()
    #     q.connector = 'AND'
    #     q.children.append(('id', request.user.id))
    #     setattr(request, 'permission_qobject', q)
    return True, msg


def user_logging_register_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_register_execl_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_operation_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_operation_execl_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def vulnerability_vuln_risk_count_get(request):
    vuln_qs = Vulnerability.objects.filter(is_delete=False, company_id=request.user.company_id)
    user_groups = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP in user_groups:
        vuln_qs_ids = vuln_qs.values_list("id", flat=True)
    else:
        subdomain_ids = get_ur_subdomain(request.user)
        vuln_qs_ids = vuln_qs.filter(subdomain_id__in=subdomain_ids).values_list("id", flat=True)
    setattr(request, "permission_qids", set(vuln_qs_ids))
    return True, "ok"


def vulnerability_vuln_risk_list_get(request):
    vuln_params = {"company_id": request.user.company_id, "is_delete": 0}

    user_groups = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP in user_groups:
        vuln_qs = Vulnerability.objects.filter(**vuln_params)
    else:
        subdomain_ids = get_ur_subdomain(request.user)
        vuln_qs = Vulnerability.objects.filter(subdomain_id__in=subdomain_ids, **vuln_params)
    setattr(request, "permission_qobject", vuln_qs)
    return True, "ok"


def vulnerability_vuln_risk_detail_get(request):
    """
    漏洞详情限制
    """
    vuln_count = 5
    # 接口限制
    try:
        if (
            request.user.company.groups.id == 1
            and Vulnerability.objects.filter(company_id=request.user.company_id, is_read=0).count() >= vuln_count
        ):
            msg = "体验用户只可查看5次漏洞详情"
            return False, msg
    except Exception as e:
        msg = "用户异常，请联系客服"
        return False, msg

    vuln_id = request.GET.get("id", None)
    if not vuln_id:
        return False, "参数不全"

    vuln_params = {"company_id": request.user.company_id, "is_delete": 0}
    user_groups = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP in user_groups:
        user_vuln_ids = Vulnerability.objects.filter(**vuln_params).values_list("id", flat=True)
    else:
        subdomain_ids = get_ur_subdomain(request.user)
        user_vuln_ids = Vulnerability.objects.filter(subdomain_id__in=subdomain_ids, **vuln_params).values_list("id", flat=True)

    if int(vuln_id) not in user_vuln_ids:
        return False, "当前资产超出查看范围，详情请联系单位管理员"
    vuln_qs = Vulnerability.objects.filter(id=int(vuln_id), **vuln_params)
    if not vuln_qs:
        return False, "数据不存在"
    setattr(request, "permission_qobject", vuln_qs)
    return True, "ok"


def vulnerability_vuln_risk_detail_put(request):
    """
    更新漏洞详情限制
    """
    vuln_params = {"company_id": request.user.company_id, "is_delete": 0}
    vuln_id = request.POST.get("id", None)
    if not vuln_id:
        return False, "参数不全"

    user_groups = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP in user_groups:
        user_vuln_ids = Vulnerability.objects.filter(**vuln_params).values_list("id", flat=True)
    else:
        subdomain_ids = get_ur_subdomain(request.user)
        user_vuln_ids = Vulnerability.objects.filter(subdomain_id__in=subdomain_ids, **vuln_params).values_list("id", flat=True)

    if int(vuln_id) not in user_vuln_ids:
        return False, "当前资产超出查看范围，详情请联系单位管理员"
    vuln_qs = Vulnerability.objects.filter(id=int(vuln_id), **vuln_params)
    if not vuln_qs:
        return False, "数据不存在"

    setattr(request, "permission_qobject", vuln_qs)
    return True, "ok"



def user_user_site_vuln_get(request):
    subdomain_ids = get_ur_subdomain(request.user)
    vuln_qs = Vulnerability.objects.filter(is_delete=False, company_id=request.user.company_id)

    user_groups = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP in user_groups:
        vuln_qs_ids = vuln_qs.values_list("id", flat=True)
    else:
        vuln_qs_ids = vuln_qs.filter(subdomain_id__in=subdomain_ids).values_list("id", flat=True)

    setattr(request, "permission_qids", set(vuln_qs_ids))
    return True, "ok"


def user_user_site_info_get(request):

    return True, ""





#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os, sys
import logging
import tornado.web
from lcyframe.libs import utils

class route(object):
    _uris = []
    _names = []
    _handlers = []
    _routes = []

    def __init__(self, uri, name=None):
        self.uri = uri
        self.name = name

    def __call__(self, _handler):
        """gets called when we class decorate"""
        self.name = self.name and self.name or _handler.__name__
        _handler.router_name = self.name

        if os.environ.app_config.get("allow_multiple_handler", True) != True:
            if self.uri in self._uris:
                logging.error("Multiple uri: %s in %s; replacing previous value" % (self.uri, _handler))
                sys.exit(0)
            if _handler.__name__ in self._handlers:
                logging.error("Multiple handlers named: %s; replacing previous value" % _handler)
                sys.exit(0)
            if self.name in self._names:
                logging.error("Multiple api named: %s in %s; replacing previous value" % (self.name, _handler))
                sys.exit(0)

        self._uris.append(self.uri)
        self._names.append(self.name)
        self._handlers.append(_handler.__name__)
        self._routes.append(tornado.web.url(self.uri, _handler, name=self.name))
        return _handler

    @classmethod
    def get_routes(cls, ROOT, handler_dir):
        if not isinstance(handler_dir, list):
            handler_dir = [handler_dir]

        for work in handler_dir:
            for root, dirs, files in os.walk(utils.fix_path(os.path.join(ROOT, work))):
                for file in files:
                    if file.startswith("__"):
                        continue
                    if file.endswith(".pyc"):
                        continue
                    if not file.endswith(".py"):
                        continue
                    model_name = root.replace(ROOT, "").lstrip("/").replace("/", ".") + "." + file.rstrip(".py")
                    __import__(model_name, globals(), locals(), [model_name], 0)
                    logging.debug("handler [%s] register success!" % model_name)
        return cls._routes

def route_redirect(from_, to, name=None):
    route._routes.append(tornado.web.url(from_, tornado.web.RedirectHandler, dict(url=to), name=name))






# coding=utf-8
import sys
import json, logging
import time
from .libs import utils
from .libs.websocket_route import WebSocketRoute
from traceback import format_exc
import asyncio, websockets
from websocket import create_connection
import inspect

"""
pip install websockets
pip install websocket-client
"""
class BaseSocket(object):
    websockets = websockets
    current_event = None
    current_conn = None
    system_uid = None

    def auth(self, conn):
        """
        校验是否合法请求
        """
        if "?" not in conn.path:
            conn.uid = ""
            return None

        path, query_string = conn.path.split("?")
        conf_path, _ = self.path.split("?")
        if path != conf_path:
            return None

        params = {item.split("=")[0]: item.split("=")[1] for item in query_string.split("&") if "=" in item}
        if "token" not in params:
            return None

        uid = params.get("uid", "")
        token = params.get("token", "")

        if token != utils.md5(uid + self.salt):
            return None

        conn.uid = uid
        conn.token = token

        self.add_client(conn)
        return True

    def packing(self, **message):
        """
        默认消息采用json编码
        """
        payload = {}
        __event__ = message.get("__event__")

        if not __event__ and not self.current_event:
            raise Exception("__event__ can not empty value")
        payload["__event__"] = __event__ or self.current_event  # message_handler name by Consumer
        payload["ts"] = int(time.time())
        payload.update(message)
        return json.dumps(payload)

    def send(self, client=None, **message):
        """
        如需要发送给指定的client，则指定client
        """
        if client is None:
            client = self.current_conn
        self.sendall(client, **message)

    def sendall(self, *client, **message):
        """
        send to all client
        默认发送给除自己及系统app以外的客户端连接
        """
        try:
            # 循环发送速度过快，容易造成网络阻塞，导致资源暂满网络缓冲区，可以用背压，没100毫秒发送一批数据
            # for ws in conns:
            #     await ws.send(message)

            # 禁止向app系统连接发送消息
            self.remove_system_client(self.conns)

            # 默认发送给所有连接（不含系统和当前消息发送方)
            if not client:
                clients = list(self.conns.values())
                if self.current_conn in clients:
                    clients.remove(self.current_conn)
            else:
                clients = list(client)

            if clients:
                websockets.broadcast(clients, self.packing(**message))
        except Exception as e:
            logging.warning(format_exc())
            logging.error(str(e))

    def client_key(self, conn):
        return f"{conn.uid}:{conn.id}"

    def add_client(self, conn):
        key = self.client_key(conn)
        if key not in self.conns:
            self.conns[key] = conn

    def remove_client(self, conn):
        key = self.client_key(conn)
        self.conns.pop(key, "")

    def remove_system_client(self, conns):
        keys = []
        for key in list(conns.keys()):
            uid, id = key.split(":")
            if uid != "" and uid in self.system_uid:
                keys.append(key)
        for key in keys:
            self.conns.pop(key, "")

    def inspect_args(self):
        import inspect
        args = inspect.getfullargspec(websockets.serve).args + inspect.getfullargspec(websockets.serve).kwonlyargs
        ws_args = {}
        for k, v in self.websocket_config.items():
            if k in args:
                ws_args[k] = v
        return ws_args

class WebSocketClient(BaseSocket):
    def __init__(self, **kwargs):
        self.host = kwargs["host"]
        self.port = kwargs["port"]
        self.system_uid = kwargs.get("system_uid", "system")    # 默认系统用户为system
        p = kwargs.pop("path", "")
        self.path = p.split("?")[0] if "?" in p else p
        self.salt = kwargs.pop("salt", "")
        self.token = utils.md5(self.system_uid + self.salt)
        self.protocol = kwargs.get("protocol", "ws")
        self.url = "%s://%s:%s%s?uid=%s&token=%s" % (self.protocol, self.host, self.port, self.path, self.system_uid, self.token)

    def conn(self):
        try:
            self.ws = create_connection(self.url)
        except Exception as e:
            logging.error(format_exc())
            logging.error("websocket connection fail")
        else:
            return self.ws

    def send(self, **message):
        self.conn()
        if self.ws.connected == True:
            self.ws.send(self.packing(**message))
            self.ws.close()
        else:
            logging.error("websocket connection closed")

    async def async_send(self, **message):
        async with websockets.connect(self.url) as websocket:
            await websocket.send(self.packing(**message))

class WebSocketWorker(BaseSocket):
    """
    websocket 服务端
    """
    def __init__(self, **kwargs):
        self.ROOT = kwargs["ROOT"]
        self.websocket_config = kwargs["websocket_config"]
        self.path = self.websocket_config.pop("path", "")
        self.protocol = self.websocket_config.pop("protocol", "ws")
        self.salt = self.websocket_config.pop("salt", "")
        self.system_uid = self.websocket_config.pop("system_uid", "system")
        self.workers_dir = self.websocket_config.pop("workers_dir", ["works/websocket"])
        self.workers = WebSocketRoute.get_workers(self.ROOT, self.workers_dir)
        self.conns = {}

    async def handler(self, websocket):
        if not self.auth(websocket):
            raise Exception("没经授权的请求：%s(%s:%s)" % (self.client_key(websocket), *websocket.remote_address[:2]))

        logging.warning("新链接:%s(%s:%s)" % (self.client_key(websocket), *websocket.remote_address[:2]))

        while True:
            try:
                message = await websocket.recv()

                # 更优的方式实现批量发送
                if message:
                    try:
                        message = json.loads(message)
                    except Exception as e:
                        logging.error(format_exc())
                        logging.error("message require JSON format")
                        await websocket.send("Error:message require JSON format")
                    event = message.get("__event__", "")
                    if not event:
                        logging.error("event can not empty value")
                        await websocket.send("Error:event can not empty value")
                    if event not in self.workers:
                        logging.error("event name %s not found!" % event)
                        await websocket.send("Error:event name %s not found!" % event)
                    else:
                        self.current_event = event
                        # websockets.broadcast(self.conns, json.dumps(message))
                        # await websocket.send(json.dumps(message))
                        self.current_conn = websocket

                        func = self.workers[event]
                        func.socket = self
                        define = inspect.getfullargspec(func)
                        if define.args:
                            self.workers[event](self, **message)
                        else:
                            self.workers[event](**message)
            except Exception as e:
                # ping的时间段内，没有收到客户端回应，将自动断开客户端连接
                if isinstance(e, (websockets.ConnectionClosedOK,
                                  websockets.ConnectionClosedError,
                                  websockets.ConnectionClosed)):
                    if self.client_key(websocket) in self.conns:
                        self.remove_client(websocket)

                    logging.warning("连接已断开：%s(%s:%s)" % (self.client_key(websocket), *websocket.remote_address[:2]))
                    # websocket.close_connection()    # 关闭当前线程连接
                    # websocket.close()               # 当连接断开，或者ping不他不通时(ping_interval)，主动关闭当前线程连接
                    break
                else:
                    logging.error(format_exc())
                    logging.error(str(e))

    async def main(self):
        # 每一个连接都单独一个线程
        async with websockets.serve(self.handler,
                                    **self.inspect_args()
                                    # "localhost", 8765,
                                    # ping_interval=20,
                                    # 一旦连接打开，每秒钟发送一个Ping帧ping_interval 。这用作保活。它有助于保持连接打开，尤其是在非活动连接上存在短暂超时的代理的情况下。设置ping_interval为None禁用此行为。
                                    # create_protocol=websockets.basic_auth_protocol_factory(
                                    #     realm="my dev server",
                                    #     credentials=("hello", "iloveyou"),
                                    # )
                                    ):
            await asyncio.Future()  # run forever


    def start(self):
        asyncio.run(self.main())

if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    kwargs = {
        "ROOT": '/Users/lcylln/www/lcyframepy3/lcyframe3-demo',
        "websocket_config": {
            "host": "127.0.0.1",
            "port": 8765,
            "workers_dir": "/Users/lcylln/www/lcyframepy3/lcyframe3-demo/works/socket"
        }}
    wk = WebSocketWorker(**kwargs)
    wk.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def get_websocket(*args, **kwargs):
    """
    websocket链接测试:
    """
    headers = {}
    files = {}
    params = {
        "nickname": "",     # string,选填,用户昵称
        }
    return send(methed="get", url="/websocket", params=params, headers=headers, files=files)


def batch_groups():
    """
    测试簇
    一键测试
    """
    get_websocket()       # websocket链接测试
    

if __name__ == "__main__":
    # websocket链接测试
    get_websocket()
    


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import logging, os, json, re
from lcyframe import BaseHandler as Handler
from lcyframe import BaseModel as Model
from lcyframe import BaseSchema as Schema
from lcyframe import utils
from utils import errors, helper, keys, constant
from SDK.minio_xa.minio_storage import MinioOpt
from collections import namedtuple

class BaseHandler(Handler):
    """
    This is the base class RequestHandler you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys
    constant = constant
    logging = logging

    def write_pagination(self, datas, counts, **kwargs):
        """
        返回列表
        :param counts: 总条数
        :param datas:
        :return:
        """
        data = {"datas": datas,
                "pagination": {"counts": counts,
                               "page": self.params.get("page", 1),
                               "count": self.params.get("count", 10),
                               }}
        if "pages" in kwargs:
            data["pagination"]["counts"] = kwargs.pop("pages", 0)

        else:
            pages = counts / self.params.get("count", 10)
            remainder = counts % self.params.get("count", 10)
            if remainder:
                pages += 1

            data["pagination"]["pages"] = pages

        data.update(kwargs)
        self.write_success(data)

    # @property
    # def gridfs(self):
    #     return GridFS(self.mongo, collection="wenshu")

    @property
    def base_data(self):
        return os.path.join(self.app_config["ROOT"], self.app_config["data_config"]["data_base"])


    @property
    def minio_html(self):
        config = os.environ.app_config
        mino_config = config["MINIO"]
        MINIO_HOST = mino_config["HOST"]
        MINIO_PORT = mino_config["PORT"]
        MINIO_ACCESS_KEY = mino_config["ACCESS_KEY"]
        MINIO_SECRET_KEY = mino_config["SECRET_KEY"]
        return MinioOpt(MINIO_HOST, MINIO_PORT, MINIO_ACCESS_KEY, MINIO_SECRET_KEY)

class BaseModel(Model):
    """
    This is the base class Model you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys
    constant = constant
    logging = logging
    gridfs = None
    es = None

    @classmethod
    def minio_html(cls):
        config = os.environ.app_config
        mino_config = config["MINIO"]
        MINIO_HOST = mino_config["HOST"]
        MINIO_PORT = mino_config["PORT"]
        MINIO_ACCESS_KEY = mino_config["ACCESS_KEY"]
        MINIO_SECRET_KEY = mino_config["SECRET_KEY"]
        return MinioOpt(MINIO_HOST, MINIO_PORT, MINIO_ACCESS_KEY, MINIO_SECRET_KEY)

class BaseSchema(Schema):

    @classmethod
    def dict_to_object(cls, dict=None):
        dict = dict or vars(cls())
        return namedtuple("Object", dict.keys())(**dict)

    @classmethod
    def shard_rule(cls, shard_key_value):
        """
        :param shard_key_value:
        :return: table_value

        This is default rule by `mod10`
        you can rewrite like this

        :example :: DemoSchema.yml
            def shard_rule(shard_key_value):
                do_your_thing
                ...
        """
        return cls.mod10(shard_key_value)

    @classmethod
    def __parse_oid(cls, d):
        for k in ["create_at", "update_at", "start_at", "end_at"]:
            prefix, _ = k.split("_")
            if k in d:
                if isinstance(d[k], str) and d[k].isdigit():
                    d[k] = int(d[k])
                d["%s_date" % prefix] = utils.timestamp2str(d[k], "%Y-%m-%d %H:%M:%S")
        return Schema.__parse_oid(d)

    @classmethod
    def create_data(cls, *args, **kwargs):
        docs = cls.fields()
        for k, v in kwargs.items():
            if k not in docs:
                continue
            docs[k] = v

        # 转为`field_name`兼容保留关键字
        COMPATIBLE_docs = {}
        for k, v in docs.items():
            COMPATIBLE_docs[f"`{k}`"] = v
        data = BaseModel.mysql.insert(cls.collection, COMPATIBLE_docs)
        if data and hasattr(cls, "_parse_data"):
            return cls._parse_data(data)
        else:
            return data

    @classmethod
    def update_data(cls, id, **kwargs):
        kwargs.pop("create_time", "")
        kwargs["update_time"] = utils.datetime.now()
        sql_condition, sql_params = cls.get_value_symbol(kwargs, "set")
        sql = f"update `{cls.collection}` set {sql_condition} where id=%s"
        sql_params.append({id})
        return BaseModel.mysql.update(sql, sql_params)

    @classmethod
    def get_data_by_kv(cls, *args, **kwargs):
        """
        单条记录
        仅支持key=value的简单形式查询(即and)，不支持or语句
        a=1,b=1,c={"in":[1, 2, 3]}
        :return:
        :rtype:
        """
        values = ",".join(args) or "*"
        sql_cond, sql_params = cls.get_value_symbol(kwargs)
        sql = f"select {values} from `{cls.collection}` where {sql_cond}"
        data = BaseModel.mysql.select_one(sql, sql_params)
        if data and hasattr(cls, "_parse_data"):
            return cls._parse_data(data)
        else:
            return data

    @classmethod
    def get_batch_by_in(cls, *args, **kwargs):
        """
        基于in的方式，查询指定键值在提供的数组内的记录。
        :return:
        :rtype:
        ref: 查询id在数组[1, 2, 3]内的记录，并返回id，name字段
        ... get_batch_by_in(id, name, id=[1, 2, 3])
        """
        field = kwargs.pop("field", "")
        return_structure = kwargs.pop("return_structure", "dict")
        # values = ",".join(args) or "*"
        # sql_cond, sql_params = cls.get_value_symbol(kwargs)
        # sql = f"select {values} from `{cls.collection}` where {sql_cond}"
        if len(kwargs) > 1:
            raise Exception("one condition parameter is required, you give %s" % len(kwargs))
        key, in_values = list(kwargs.items())[0]
        field = field or "id"
        in_values = list(in_values) if not isinstance(in_values, (list, tuple)) else in_values
        kw = {}
        kw["values"] = ["*"] if not args else list(set([field] + list(args)))
        kw["sql_and"] = {key: {"in": list(set(in_values))}}
        sql, sql_params = cls.get_sql_and_params(**kw)
        datas = BaseModel.mysql.select_many(sql, sql_params)
        if return_structure == "list":
            return [cls._parse_data(data) for data in datas] if datas else []
        else:
            return {data[field]: cls._parse_data(data) for data in datas} if datas else {}

    @classmethod
    def get_data_by_spec(cls, *args, **kwargs):
        """
        查询单条数据
        args： 需要返回的字段列表
        kwargs参数含义：
        sql_and： and条件
            {"a":1, "b":-1, "c": {"in": [1, 2, 3]}}
        sql_or: or条件
            {"x":1, "y":1}
        orderby： 排序
            {"a":1, "b":-1} a升序，b将序，无序
            [("a", 1), ("b", -1)] a升序，b将序
        page: 页码skip-1
        count: 每页数limit
        """
        kwargs["values"] = args
        sql, params = cls.get_sql_and_params(**kwargs)
        data = BaseModel.mysql.select_one(sql, params)
        if data and hasattr(cls, "_parse_data"):
            return cls._parse_data(data)
        else:
            return data

    @classmethod
    def get_datas_by_spec(cls, *args, **kwargs):
        """
        查询多条数据
        args： 需要返回的字段列表
        kwargs参数含义：
        sql_and： and条件
            {"a":1, "b":-1, "c": {"in": [1, 2, 3]}}
        sql_or: or条件
            {"x":1, "y":1}
        orderby： 排序
            {"a":1, "b":-1} a升序，b将序，无序
            [("a", 1), ("b", -1)] a升序，b将序
        page: 页码skip-1
        count: 每页数limit
        """
        kwargs["values"] = args
        sql, params = cls.get_sql_and_params(**kwargs)
        datas = BaseModel.mysql.query_sql(sql, params)
        if datas and hasattr(cls, "_parse_data"):
            return [cls._parse_data(data) for data in datas]
        else:
            return datas


    @classmethod
    def get_list_by_page(cls, page, count, **kwargs):
        """
        分页
        kwargs参数含义：
        sql_and： and条件
            {"a":1, "b":-1, "c": {"in": [1, 2, 3]}}
        sql_or: or条件
            {"x":1, "y":1}
        orderby： 排序
            {"a":1, "b":-1} a升序，b将序，无序
            [("a", 1), ("b", -1)] a升序，b将序
        page: 页码skip-1
        count: 每页数limit
        """
        orderby = kwargs.pop("orderby", [])
        sql, params = cls.get_sql_and_params(**kwargs)
        counts = cls.get_data_counts(sql, params)

        kwargs["orderby"] = orderby
        kwargs["page"] = page
        kwargs["count"] = count
        sql, params = cls.get_sql_and_params(**kwargs)
        # orderby = orderby if isinstance(orderby, (list, tuple)) else orderby.items()
        # items = ["`%s` %s" % (k, "ASC" if v >= 1 else "DESC") for k, v in orderby]
        # orderby_condition = ",".join(items) if items else ""
        # if orderby_condition:
        #     sql += " order by " + orderby_condition
        # limit = f" limit {(page - 1) * count}, {count}"
        # sql += limit
        datas = BaseModel.mysql.query_sql(sql, params)
        if datas and hasattr(cls, "_parse_data"):
            datas = [cls._parse_data(data) for data in datas]
        return datas, counts

    @classmethod
    def get_data_counts(cls, sql, params=None):
        total_count = BaseModel.mysql.select_many(sql, params)
        counts = 0 if not total_count else len(total_count)
        return counts

    @classmethod
    def get_sql_and_params(cls, **kwargs):
        """
        组装sql基础语句
        kwargs:
            ... sql='select * from user where  company_id=%s and id=%s', params=[1, 33]

            ... sql_and={'company_id': 1, 'id': 33}
            ... sql_and={'company_id': 1, 'id': [">=", 33]}, orderby={'id': 1, create_at: -1}
            ... sql_and={'company_id': 1, 'id': ["in", (33, 34)]}, orderby={'id': 1, create_at: -1}
            ... sql_and={'company_id': 1, 'create_time': {"between": [start_time, end_time]}}
            ... sql_and={'company_id': 1, 'create_time': ["between", (start_time, end_time)]}
            ... sql_and={'company_id': 1, 'create_time': {">": start_time, "<=": end_time}}
            ... sql_and={'company_id': 1, 'create_time': [(">", start_time), ("<=", end_time)}

            ... sql_or={'company_id': 1, 'gid': ["in", (1, 2)]}
            ... sql_or={'account': {"REGEXP": self.params.get("search"),
                        'email': ["REGEXP", self.params.get("search")]
                    }

            ... orderby={'id': 1, create_at: -1}

        If args is a list or tuple, %s can be used as a placeholder in the query.
        If args is a dict, %(name)s can be used as a placeholder in the query.
        values: 返回字段
            values = fields1,fields2,...
            values = [fields1,fields2,...]
        and条件：
            {"a":1, "b":2} 不关心查询顺序
            [("b", 2), ("a", 1)] 按顺序组装查询语句
        or条件: 推荐组装好sql在传入，例
            select * from user where a=1 and id=33 or id=28 ==> (a=1 and id=33) or id=28
            select * from user where a=1 and (id=33 or id=28) ==> (a=1 and id=33) or (a=1 and id=28)
        """
        params = kwargs.pop("params", [])
        values = kwargs.pop("values", [])
        sql = kwargs.pop("sql", "")
        if params and not sql:
            raise Exception("sql must be provided, if you given params")
        if not sql:
            if values:
                values = cls.format_fields_name(values)
                values = values if isinstance(values, str) else ",".join(values)
            else:
                values = "*"
            sql = f"select {values} from `{cls.collection}`"

        and_condition = or_condition = ""
        # and
        sql_and = kwargs.pop("sql_and", [])
        if sql_and:
            and_condition, and_symbol_params = cls.get_value_symbol(sql_and, "and")
            params.extend(and_symbol_params)

        # or
        sql_or = kwargs.pop("sql_or", [])
        if sql_or:
            or_condition, or_symbol_params = cls.get_value_symbol(sql_or, "or")
            or_condition = (" (" + or_condition + ") ") if or_condition else or_condition
            params.extend(or_symbol_params)

        # groupby
        groupby = kwargs.pop("groupby", [])
        if groupby:
            items = ["`%s`" for _ in groupby]
            groupby_condition = ",".join(items) if items else ""
            if groupby_condition:
                params.extend(groupby)
                if cls.filter_sql_illegalchar(",".join(groupby)):
                    raise Exception("You have an error in your SQL syntax; near groupby '%s'" % ",".join(groupby))
        else:
            groupby_condition = ""

        # orderby
        orderby = kwargs.pop("orderby", [])
        if orderby:
            orderby = orderby if isinstance(orderby, (list, tuple)) else orderby.items()
            items = ["`%s` %s" % (k, "ASC" if v >= 1 else "DESC") for k, v in orderby]
            orderby_condition = ",".join(items) if items else ""
            if cls.filter_sql_illegalchar(orderby_condition):
                raise Exception("You have an error in your SQL syntax; near orderby '%s'" % orderby_condition)
        else:
            orderby_condition = ""

        # skip\limit
        page = kwargs.pop("page", "")
        count = kwargs.pop("count", "")
        if page and count:
            limit_condition = f" {(page - 1) * count}, {count}"
        elif count:
            limit_condition = f" {count}"
        else:
            limit_condition = ""

        # sql
        if and_condition or or_condition:
            if and_condition and or_condition:
                sql += " where " + " and ".join([and_condition, or_condition])
            elif and_condition:
                sql += " where " + and_condition
            else:
                sql += " where " + or_condition
        if groupby_condition:
            sql += " group by " + groupby_condition
        if orderby_condition:
            sql += " order by " + orderby_condition
        if limit_condition:
            sql += " limit " + limit_condition

        return sql, params

    @classmethod
    def get_value_symbol(cls, condition_data, sql_type="and"):
        def __symbol_str(condition, symbol_str, sql_params):
            if isinstance(condition, (list, tuple)):
                if len(condition) == 1:
                    symbol = "="
                    value = condition
                else:
                    symbol = f"{condition[0]}"
                    value = condition[1]
            else:
                symbol = "="
                value = condition

            if symbol.lower() == "=":
                symbol_str.append(f"`{k}`{symbol}%s")
                sql_params.append(value)
            elif symbol.lower() == "between":
                symbol_str.append(f"`{k}` {symbol} %s and %s")
                sql_params.append(value[0])
                sql_params.append(value[1])
            elif symbol.lower() in ["in"]:
                if value:
                    symbol_str.append(f"`{k}` {symbol} %s")
                    sql_params.append(tuple(value))
            else:
                symbol_str.append(f"`{k}` {symbol} %s")
                sql_params.append(value)

            return symbol_str, sql_params

        condition_data = condition_data if isinstance(condition_data, (list, tuple)) else condition_data.items()
        symbol_str = []
        sql_params = []
        for k, items in condition_data:
            if isinstance(items, (dict,)):
                for item in items.items():
                    symbol_str, sql_params = __symbol_str(item, symbol_str, sql_params)
            elif isinstance(items, (list, tuple, set)):
                if isinstance(items[0], (str, bytes)):
                    symbol_str, sql_params = __symbol_str(items, symbol_str, sql_params)
                else:
                    for item in items:
                        symbol_str, sql_params = __symbol_str(item, symbol_str, sql_params)
            else:
                symbol_str, sql_params = __symbol_str(items, symbol_str, sql_params)
        if sql_type == "set":
            sql_condition = f",".join(symbol_str) if symbol_str else ""
        else:
            sql_condition = f" {sql_type} ".join(symbol_str) if symbol_str else ""
        # return sql_condition, sql_params if len(sql_params) > 1 else sql_params[0] if len(sql_params) == 1 else sql_params
        return sql_condition, sql_params

    @classmethod
    def format_datetime(cls, docs, fields=None):
        """
        格式化时间
        """
        import datetime, re
        if fields:
            datetime_key = fields
        else:
            datetime_key = docs.keys()
        for k in datetime_key:
            if k not in docs:
                continue
            if type(docs[k]) == datetime.datetime:
                docs[k] = utils.datetime2string(docs[k])
            else:
                if type(docs[k]) in [bytes, str]:
                    if re.match("^2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}", docs[k]):
                        try:
                            date = utils.string2datetime(docs[k], "%Y-%m-%d %H:%M:%S.%f")
                            docs[k] = utils.datetime2string(date)
                        except Exception as e:
                            date = utils.string2datetime(docs[k], "%Y-%m-%d %H:%M:%S")
                            docs[k] = utils.datetime2string(date)
                        else:
                            continue
                    else:
                        continue

        return docs

    @classmethod
    def format_fields_name(cls, fields):
        """
        字段名称加上``，避免误用保留字导致sql语句错误
        """
        if isinstance(fields, str):
            return "*" if "*" in fields else f"`{fields}`"
        else:
            symbol_fields = []
            if "*" in fields:
                symbol_fields = ["*"]
            else:
                for k in fields:
                    symbol_fields.append(f"`{k}`")
            return symbol_fields

    @classmethod
    def filter_sql_illegalchar(cls, sql_str):
        illegalchars = ["union", "sleep", "select", "load_file", "extractvalue", "updatexml", "regexp"]
        return re.findall("|".join(illegalchars), sql_str, re.I)


# -*- coding:utf-8 -*-
# 1~n级为标签，不具备权限值。只有所有子权限都被勾选后，当前标签勾选，否则不勾选。标签允许有任意多个上级和下级。
# 倒数第二级为API级别权限，称为接口权限。key为handler的名称url_name.拦截位置处于系统顶层入口处（只针对permission_class=api的权限），不侵入业务（可做到完全解耦）。当子类（如有）任意权限被勾选时，当前权限勾选。否则不勾选
# 倒数第一级为逻辑权限，是最小粒度的权限类型。属于API级的子类权限且不能再派生出子类权限。缺点是需要侵入业务内，即耦合较强。
# API权限辅助钩子，位于API数据体内的hook_func所制定的函数。用来拓展或者增强API接口的权限拦截，实现更高的定制化配置（如当请求参数里包含什么值时，禁止访问或修正该值）。
# 它不同于逻辑权限之处在于，钩子适合只做简单的条件判断；逻辑权限可在代码任意位置、任意逻辑、任意即时数据的值进行更复杂的进行判断。包括条件依赖判断、数据依赖判断、关系依赖判断。甚至在for循环内进行实时判断。
# key:method必须全局唯一，且一单设置永不得修改（除非用脚本全量修改且更新代码内的handler.url_name)，如有需要请通过删除该条struct,在用新的key定义新的struct
"""
要求：
1、api接口仅支持get、post、put、方法，且分别对应查、增、改、删
2、api权限一对一，一个key只对应一个url请求。
3、api权限一对多，一个key对应若干个url请求，如
    1）url采用动态路由设置时（/(thumb|comment)/detail代表2个url详情）,key=thumb_thumb,method=get可以实现权限："允许查看点赞|评论数据"权限，
    2）url采用动态路由设置时（/(thumb|comment)代表2个url详情），key=thumb_thumb,method=get,post,put,可以实现权限："允许增删查改点赞|评论数据"权限，
    3）使用逻辑权限

权限表结构设置:
{
    "name": "允许查看和新增",             # 权限名称
    "key": "control2-xxx-api",          # 路由名称,取url_name的值
    "method": "get,post,可空",           # 该权限支持的方法,多个用逗号隔开, 仅针对permission_class=api时有效
    "parent_id": 0,                      # 父级的id值，parent和parent_id二选一
    # "parent": 0,                        # 父级的key或key:method值的组合，为空时代表顶级
    "permission_class": "",                        # 类别，tag标签；api接口权限；logic逻辑权限
    "hook_func": "",                    # 钩子，可空
    "state": 1,                         # 1启用0禁用，默认1
    "desc": "",                         # 详细说明
    "url": "",                          # 权限对应的url，可空
    "content_type": 1 or 0              # 1 业务端使用，0系统管理端
}
角色表：
groups：{
    "name": "角色名称",
    #                       # 父级id,暂不支持
    # "group_class": "技术部",            # 分类，可以按标签|部门|组织等自定义划分
    "state": 1,                           # 1可以 0 禁用 默认1
    "content_type": 1 or 0,             # 1 业务端使用，0系统管理端
    "permissions": {
        "control2-xxx-api:get": 1,
        "control2-xxx-api2:get,post,put,delete": 1,
        "monitoring-tasks-api:post": 1
    }
}
商户角色：分配给商户的角色
company_groups: {
    "group_id": "group.id",
    "company_id": "",
}

例：permissions: []
标签1：{"name": "控制台", "key": "control1", "parent": "无父级，留空", "permission_class": "tag", "state": "1启用0停用，默认1", "desc": "详细说明，可空"}
    标签2: {"name": "左侧区域", "key": "control2", "parent": "control1;父级的id值,不填时有程序自动识别", "permission_class": "tag"}
        API权限1: {"name": "查看详情", "key": "control2-xxx-api", "method": "get", "parent": "control2;父级的id值", "permission_class": "api", "hook_func": "func_for_definition"}
            逻辑权限1: {"name": "男性能查看游戏数据", "key": "control2-xxx-api-for-boy", "method": "get", "parent": "control2-xxx-api;父级的id值", "permission_class": "logic"}
            逻辑权限2: {"name": "女性能查看购物数据", "key": "control2-xxx-api-for-girls", "method": "get", "parent": "control2-xxx-api;父级的id值", "permission_class": "logic"}
            逻辑权限3: {"name": "查看点赞|评论数据", "key": "control2-thumb-comment", "method": "get", "parent": "control2-xxx-api;父级的id值", "permission_class": "logic"}
            逻辑权限4: {"name": "增删查改点赞|评论数据", "key": "control2-thumb-comment", "method": "get,post,put,", "parent": "control2-xxx-api;父级的id值", "permission_class": "logic"}
        API权限2: {"name": "添加数据", "key": "control2-xxx-api", "method": "post", "parent": "control2;父级的id值", "permission_class": "api", "hook_func": "func_for_definition，默认空"}
        API权限3: {"name": "删除数据", "key": "control2-xxx-api", "method": "", "parent": "control2;父级的id值", "permission_class": "api", "hook_func": "func_for_definition"}
    标签3: {"name": "右侧区域", "key": "control3", "parent": "control1;父级的id值", "permission_class": "tag"}
    ...
    API权限4: {"name": "查看", "key": "url_name", "parent": "control1;父级的id值", "permission_class": "tag"}
    API权限5: {"name": "查看", "key": "url_name", "parent": "control1;父级的id值", "permission_class": "tag"}
        逻辑权限a
        逻辑权限b
    ...
    逻辑权限i
    逻辑权限j
    ...
标签4
标签5
...
API权限6
API权限7
...
逻辑权限x
逻辑权限y

角色权限结构:
group_permission1：{
    "control2-xxx-api:get": 1代表有权限，0或不存在该key，代表无权限,
    "control2-xxx-api-for-girls:get": 0
    "control2-xxx-api:get,post": 1, # 组合权限，允许新增和查看。不能按方法分拆为多条，否则无法映射回原始权限条目
    "control2-xxx-api:get,post": 1, # 组合权限，允许新增和查看。不能按方法分拆为多条，否则无法映射回原始权限条目
}
group_permission2：{
    "control2-xxx-api:get": 1代表有权限，0或不存在该key，代表无权限,
    "control2-xxx-api-for-girls:get": 0
}

配置步骤：
1、编写脚本将数据生成类型下面的permission结构，
2、用导入脚本将permission结构转为单条db数据，key+method组合唯一，写入db或redis
3、在管理界面调用，读出permission数据渲染
4、按角色分别勾选权限后保存，将被勾选的权限key保存到角色内
5、修改权限：导出数据并组装为permission结构，进行修改；重复第2步。或者直接在线上修改，然后导出备份
"""

# 业务端权限
service_permissions = [
    {"name": "控制台", "key": "control1", "permission_class": "tag",
     "children": [
        {"name": "左侧区域", "key": "control2", "permission_class": "tag", "parent": "父级的id值,不填时有程序自动识别",
         "children": [
            {"name": "查看详情", "key": "user_detail", "method": "get", "permission_class": "api", "hook_func": "func_for_definition，默认空",
             "children": [
                # 暂不支持逻辑权限
                # {"name": "男性能查看游戏数据", "key": "ontrol2-xxx-api", "permission_class": "logic"},
                # {"name": "女性能查看购物数据", "key": "ontrol2-xxx-api", "permission_class": "logic"}
             ]}
         ]},
        {"name": "右侧区域", "key": "control2", "parent": "control1", "permission_class": "tag", "children": []}
    ]},
    {"name": "监测任务", "key": "monitoring-tasks", "permission_class": "tag", "children": [
        {"name": "查看", "key": "monitoring-tasks-api", "method": "get", "permission_class": "api", "hook_func": ""},
        {"name": "新增", "key": "monitoring-tasks-api", "method": "post", "permission_class": "api"},
        {"name": "编辑", "key": "monitoring-tasks-api", "method": "put", "permission_class": "api"},
        {"name": "删除", "key": "monitoring-tasks-api", "method": "delete", "permission_class": "api"},
        {"name": "停用/启用", "key": "monitoring-tasks-state", "method": "get", "permission_class": "api"},
    ]},
    {"name": "风险监测", "key": "monitoring-risk", "permission_class": "tag", "children": [
        {"name": "敏感内容", "key": "monitoring-risk-stc", "permission_class": "tag", "children": [
            {"name": "查看", "key": "monitoring-risk-stc", "method": "get", "permission_class": "api", "hook_func": ""},
            {"name": "新增", "key": "monitoring-risk-stc", "method": "post", "permission_class": "api"},
            {"name": "编辑", "key": "monitoring-risk-stc", "method": "put", "permission_class": "api"},
            {"name": "删除", "key": "monitoring-risk-stc", "method": "delete", "permission_class": "api"},
            {"name": "导出", "key": "monitoring-risk-stc-export", "method": "get", "permission_class": "api"},
            {"name": "导入", "key": "monitoring-tasks-stc-import", "method": "post", "permission_class": "api"},
        ]},
        {"name": "网页篡改", "key": "monitoring-risk-wtd", "permission_class": "tag", "children": [
            {"name": "查看", "key": "monitoring-risk-wtd", "method": "get", "permission_class": "api", "hook_func": ""},
            {"name": "新增", "key": "monitoring-risk-wtd", "method": "post", "permission_class": "api"},
            {"name": "编辑", "key": "monitoring-risk-wtd", "method": "put", "permission_class": "api"},
            {"name": "删除", "key": "monitoring-risk-wtd", "method": "delete", "permission_class": "api"},
            {"name": "导出", "key": "monitoring-risk-wtd-export", "method": "get", "permission_class": "api"},
            {"name": "导入", "key": "monitoring-tasks-wtd-import", "method": "post", "permission_class": "api"},
        ]},
        {"name": "挂马检测", "key": "monitoring-risk-trj", "permission_class": "tag", "children": [
            {"name": "查看", "key": "monitoring-risk-trj", "method": "get", "permission_class": "api", "hook_func": ""},
            {"name": "新增", "key": "monitoring-risk-trj", "method": "post", "permission_class": "api"},
            {"name": "编辑", "key": "monitoring-risk-trj", "method": "put", "permission_class": "api"},
            {"name": "删除", "key": "monitoring-risk-trj", "method": "delete", "permission_class": "api"},
            {"name": "导出", "key": "monitoring-risk-trj-export", "method": "get", "permission_class": "api"},
            {"name": "导入", "key": "monitoring-tasks-trj-import", "method": "post", "permission_class": "api"},
        ]},
        {"name": "暗链检测", "key": "monitoring-risk-bhs", "method": "get", "permission_class": "tag", "children": [
            {"name": "查看", "key": "monitoring-risk-bhs", "method": "get", "permission_class": "api", "hook_func": ""},
            {"name": "新增", "key": "monitoring-risk-bhs", "method": "post", "permission_class": "api"},
            {"name": "编辑", "key": "monitoring-risk-bhs", "method": "put", "permission_class": "api"},
            {"name": "删除", "key": "monitoring-risk-bhs", "method": "delete", "permission_class": "api"},
            {"name": "导出", "key": "monitoring-risk-bhs-export", "method": "get", "permission_class": "api"},
            {"name": "导入", "key": "monitoring-tasks-bhs-import", "method": "post", "permission_class": "api"},
        ]}
    ]},
    {"name": "报告管理", "key": "report-manage", "permission_class": "tag", "children": [
        {"name": "查看", "key": "report-manage", "method": "get", "permission_class": "api", "hook_func": ""},
        {"name": "新增", "key": "report-manage", "method": "post", "permission_class": "api"},
        {"name": "编辑", "key": "report-manage", "method": "put", "permission_class": "api"},
        {"name": "删除", "key": "report-manage", "method": "delete", "permission_class": "api"},
        {"name": "导出", "key": "report-manage-export", "method": "get", "permission_class": "api"},
        {"name": "导入", "key": "report-manage-import", "method": "post", "permission_class": "api"},
    ]},
    {"name": "资源配置", "key": "resource-manage", "permission_class": "tag", "children": []},
    {"name": "资产配置", "key": "asset-manage", "permission_class": "tag", "children": []},
]

# 系统管理端权限
system_permissions = []

# 业务端角色
service_groups = [
        {"name": "网站管理员", "desc": "主账号"},
        {"name": "网站运营人员", "desc": "安全管理员"},
        {"name": "普通用户", "desc": "数据分析师、运营人员等"},
        {"name": "体验用户", "desc": ""},
]
# 系统管理端角色
system_groups = [
        {"name": "系统管理员", "desc": ""},
        {"name": "管理员", "desc": ""},
        {"name": "研发人员", "desc": ""},
        {"name": "测试人员", "desc": ""},
]

if __name__ == '__main__':
    pass



    
import pathlib, importlib
import time

test_path = pathlib.Path(__file__).parent
# print(test_path)

print("开始一键测试: %s" % test_path)
time.sleep(1)

for path in test_path.rglob('*.py'):
    filename = path.name
    module = path.stem
    if module in [test_path.name, "__init__"]:
        continue
    # print(filename, module)
    import_object = importlib.import_module(module)
    if not hasattr(import_object, "batch_groups"):
        print("正在执行脚本: %s" % filename)
        raise Exception("文件[%s]，未找到测试簇方法:batch_groups()" % filename)

    # 执行测试簇
    print("正在执行脚本: %s" % filename)
    try:
        getattr(import_object, 'batch_groups')()
    except Exception as e:
        print(str(e))
        print("测试退出")
        exit()
    else:
        print("测试完成")
    


#!/usr/env/bin python
# -*- coding:utf-8 -*-

class Copyright(object):
    frame_name = "LcyFrame"
    auto_name = "Lcylln"
    auto_email = "123220663@qq.com"

    # file_pre = "# /usr/env/bin python\n# -*- coding:utf-8 -*-\n# Copyright %s %s" % (auto_name, auto_email)
    # file_pre = "#! /usr/env/bin python\n# -*- coding:utf-8 -*-"
    file_pre = ""   # p3




#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def post_authorization(*args, **kwargs):
    """
    签发授权:授权开通使用某个服务、套餐。开通后，分自动激活和客户邮件连接激活
    """
    headers = {}
    files = {}
    params = {
        "company_id": 0,     # int,必填,客户id
        "product_id": 0,     # int,必填,产品id
        "service_life": 0,     # int,必填,授权时长，即产品有效期，单位天
        "domain_name_list": "",     # json,必填,授权主域列表
        "active_mode": 0,     # int,必填,激活方式, 0直接激活，1邮件激活
        "email": "",     # str,选填,授权邮箱
        }
    return send(methed="post", url="/authorization", params=params, headers=headers, files=files)

def get_authorization(*args, **kwargs):
    """
    详情:
    """
    headers = {}
    files = {}
    params = {
        "id": 0,     # int,必填,id
        }
    return send(methed="get", url="/authorization", params=params, headers=headers, files=files)

def put_authorization(*args, **kwargs):
    """
    编辑:
    """
    headers = {}
    files = {}
    params = {
        "id": 0,     # int,必填,记录id
        "service_list": "",     # json,选填,服务条目
        "domain_name_list": "",     # json,选填,授权域名
        "service_end": "",     # datetime,选填,到期时间
        }
    return send(methed="put", url="/authorization", params=params, headers=headers, files=files)

def delete_authorization(*args, **kwargs):
    """
    删除:删除记录
    """
    headers = {}
    files = {}
    params = {
        "id": 0,     # int,必填,记录id
        }
    return send(methed="delete", url="/authorization", params=params, headers=headers, files=files)


def get_authorization_list(*args, **kwargs):
    """
    授权列表:
    """
    headers = {}
    files = {}
    params = {
        "page": 0,     # integer,选填,翻页码
        "count": 0,     # integer,选填,每页显示条数
        "auth_state": 0,     # int,选填,授权状态：0待激活，1已激活，2已过期
        "search": "",     # str,选填,搜索授权客户名称或产品名称等关键字查询
        "order_key": "",     # str,选填,排序字段,service_end=-1
        }
    return send(methed="get", url="/authorization/list", params=params, headers=headers, files=files)


def batch_groups():
    """
    测试簇
    一键测试
    """
    post_authorization()       # 签发授权
    get_authorization()       # 详情
    put_authorization()       # 编辑
    delete_authorization()       # 删除
    get_authorization_list()       # 授权列表
    

if __name__ == "__main__":
    # 签发授权
    post_authorization()
    # 详情
    get_authorization()
    # 编辑
    put_authorization()
    # 删除
    delete_authorization()
    # 授权列表
    get_authorization_list()
    


# -*- coding: utf-8 -*-
"""
腾讯云
"""
import base64
from configs import config_tencentcloud

from tencentcloud.common import credential
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.iai.v20180301 import iai_client, models

config = config_tencentcloud.get()


# TODO 先安装，后使用：pip install --upgrade tencentcloud-sdk-python

class TencentCloud():
    cred = credential.Credential(config["secretid"], config["secretkey"])

class Iai(TencentCloud):
    """
    人脸识别相关API
    """
    client = iai_client.IaiClient(TencentCloud.cred, "ap-guangzhou")
    client._apiVersion = "2020-03-03"
    client._endpoint = "https://iai.tencentcloudapi.com"

    @classmethod
    def DetectLiveFaceRequest(cls, url=None, image=None):
        """
        人脸静态活体验证：通过后才能注册存库
        """
        req = models.DetectLiveFaceRequest()
        req.FaceModelVersion = "3.0"

        if url:
            req.Url = url
        elif image:
            req.Image = image

        try:
            resp = cls.client.DetectFaceAttributes(req)
            return resp
        except TencentCloudSDKException as e:
            return e

    @classmethod
    def CreateGroup(cls, group_id, group_name, group_desc=[], tag=""):
        """
        创建人员库
        group_id: 系统内确保唯一
        """
        req = models.CreateGroupRequest()
        req.FaceModelVersion = "3.0"

        req.GroupId = str(group_id)
        req.GroupName = str(group_name)
        req.GroupExDescriptions = list(group_desc)
        req.Tag = str(tag) if tag else None

        try:
            resp = cls.client.CreateGroup(req)
            return resp
        except TencentCloudSDKException as e:
            return e

    @classmethod
    def CreatePerson(cls, group_id, person_name, person_id, gender=0, url=None, image=None, UniquePersonControl=2):
        """
        创建人员
        person_id: 内部user_id，确保唯一
        image: 图片 base64 数据，base64 编码后大小不可超过5M。
        jpg格式长边像素不可超过4000，其他格式图片长边像素不可超2000。
        支持PNG、JPG、JPEG、BMP，不支持 GIF 图片。
        url: 图片的 Url 。对应图片 base64 编码后大小不可超过5M。
        jpg格式长边像素不可超过4000，其他格式图片长边像素不可超2000。
        Url、Image必须提供一个，如果都提供，只使用 Url。
        """
        req = models.CreatePersonRequest()

        req.GroupId = str(group_id)
        req.PersonName = str(person_name)
        req.PersonId = str(person_id)
        req.Gender = int(gender)
        req.UniquePersonControl = int(UniquePersonControl) if UniquePersonControl else 0

        if url:
            req.Url = url
        elif image:
            req.Image = image

        try:
            resp = cls.client.CreatePerson(req)
            return resp
        except TencentCloudSDKException as e:
            return e

    @classmethod
    def VerifyFaceRequest(cls, person_id, url=None, image=None):
        """
        人脸验证：用于确认本次上传图与刚才上传的是否为同一人

        person_id: 内部user_id，确保唯一
        image: 图片 base64 数据，base64 编码后大小不可超过5M。
        jpg格式长边像素不可超过4000，其他格式图片长边像素不可超2000。
        支持PNG、JPG、JPEG、BMP，不支持 GIF 图片。
        url: 图片的 Url 。对应图片 base64 编码后大小不可超过5M。
        jpg格式长边像素不可超过4000，其他格式图片长边像素不可超2000。
        Url、Image必须提供一个，如果都提供，只使用 Url。
        """
        req = models.VerifyFaceRequest()
        req.PersonId = str(person_id)

        if url:
            req.Url = url
        elif image:
            req.Image = image

        try:
            resp = cls.client.VerifyFace(req)
            return resp
        except TencentCloudSDKException as e:
            return e

    @classmethod
    def SearchFacesRequest(cls, group_id, url=None, image=None):
        """
        人脸搜索：用于刷脸登录，每次提供一张图片，返回1条最匹配的人员信息
        """
        req = models.SearchFacesRequest()
        req.GroupIds = [str(group_id)]
        if url:
            req.Url = url
        elif image:
            req.Image = image

        try:
            resp = cls.client.SearchFaces(req)
            # {"PersonId": "0", "FaceId": "4100131568524211187", "Score": 100, "PersonName": null, "Gender": null, "PersonGroupInfos": null}
            return resp.Results[0].Candidates[0]
        except TencentCloudSDKException as e:
            return e
        
if __name__ == "__main__":

    # resp = Iai.DetectLiveFaceRequest("https://gimg2.baidu.com/image_search/src=http%3A%2F%2Fattachbak.dataguru.cn%2Fattachments%2Fportal%2F201812%2F29%2F161729gqqfq4qa4oli51oi.jpg&refer=http%3A%2F%2Fattachbak.dataguru.cn&app=2002&size=f9999,10000&q=a80&n=0&g=0n&fmt=jpeg?sec=1619580553&t=a773a07cdaabae7977cf8434278eeacf")
    # resp = Iai.CreateGroup(2, "我是人员库名", ["描述1", "描述2", "描述3"], "我是tag")
    # resp = Iai.CreatePerson(2, "我是人名", "0", url="https://gimg2.baidu.com/image_search/src=http%3A%2F%2Fattachbak.dataguru.cn%2Fattachments%2Fportal%2F201812%2F29%2F161729gqqfq4qa4oli51oi.jpg&refer=http%3A%2F%2Fattachbak.dataguru.cn&app=2002&size=f9999,10000&q=a80&n=0&g=0n&fmt=jpeg?sec=1619580553&t=a773a07cdaabae7977cf8434278eeacf")
    # resp = Iai.VerifyFaceRequest("0", url="https://gimg2.baidu.com/image_search/src=http%3A%2F%2Fattachbak.dataguru.cn%2Fattachments%2Fportal%2F201812%2F29%2F161729gqqfq4qa4oli51oi.jpg&refer=http%3A%2F%2Fattachbak.dataguru.cn&app=2002&size=f9999,10000&q=a80&n=0&g=0n&fmt=jpeg?sec=1619580553&t=a773a07cdaabae7977cf8434278eeacf")
    resp = Iai.SearchFacesRequest("2", url="https://gimg2.baidu.com/image_search/src=http%3A%2F%2Fattachbak.dataguru.cn%2Fattachments%2Fportal%2F201812%2F29%2F161729gqqfq4qa4oli51oi.jpg&refer=http%3A%2F%2Fattachbak.dataguru.cn&app=2002&size=f9999,10000&q=a80&n=0&g=0n&fmt=jpeg?sec=1619580553&t=a773a07cdaabae7977cf8434278eeacf")
    print(resp)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.basics_schema import BasicsSchema

class BasicsModel(BaseModel, BasicsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "_id" in doc:
            doc["basics_id"] = doc.pop("_id", "")

        return doc

    @classmethod
    def send_smscode(cls, uuid, mobile, stype, **kwargs):
        """
        发送短信验证码
        :return:
        :rtype:
        """
        minute_flag = cls.redis.get(cls.keys.smscode_interval % mobile) or 0
        if minute_flag:
            raise cls.api_error.ErrorCommon("发送太频繁，请稍后在尝试")

        day_times = cls.redis.get(cls.keys.smscode_times % mobile) or b'0'
        if int(day_times.decode()) >= cls.keys.sms_times:
            raise cls.api_error.ErrorCommon("短信验证码每个账号每天最多只能收到10条")

        sms_code = cls.utils.random_int(4)
        if stype == 1:
            content = f"【XXXX】您的短信验证码为：{sms_code}，有效期为5分钟；请勿泄露给他人，如非本人操作，请忽略此信息。"
        else:
            content = ""
        result = cls.helper.smscode(mobile, content, 4)
        if result.status_code == 200 and cls.utils.to_python(result.content)["code"] == 100:
            cls.redis.setex(cls.keys.smscode_key % uuid, cls.keys.smscode_exp, sms_code)
            cls.redis.setex(cls.keys.smscode_interval % mobile, 60, sms_code)
            cls.redis.incr(cls.keys.smscode_times % mobile)
            cls.redis.expire(cls.keys.smscode_times % mobile, cls.utils.get_today_lasttime()-cls.utils.now())
            return sms_code
        raise cls.api_error.ErrorCommon("发送失败")

    @classmethod
    def vaild_smscode(cls, uuid, code, **kwargs):
        """
        校验短信验证码
        :return:
        :rtype:
        """
        sms_code = cls.redis.get(cls.keys.smscode_key % uuid)
        sms_code = sms_code.decode() if sms_code else ""
        if code == sms_code:
            cls.redis.delete(cls.keys.smscode_key % uuid)
        return code == sms_code

    @classmethod
    def send_imagecode(cls, uuid, **kwargs):
        """
        生成图形验证码
        :return:
        :rtype:
        """
        image_code, code_data = cls.utils.imagecode(**{"lines_count": 8, "lines_width": 1, "dots_count": 100})
        cls.redis.setex(cls.keys.imagecode_key % uuid, cls.keys.imagecode_exp, image_code)
        return image_code, code_data

    @classmethod
    def vaild_imagecode(cls, uuid, code, **kwargs):
        """
        校验图形验证码
        :return:
        :rtype:
        """
        image_code = cls.redis.get(cls.keys.imagecode_exp % uuid)
        image_code = image_code.decode() if image_code else ""
        if code == image_code:
            cls.redis.delete(cls.keys.imagecode_key % uuid)
        return code == image_code


    @classmethod
    def get_basics_list_by_last_id(cls, last_id, count, **kwargs):
        """
        前端列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=kwargs.get("fields", None),
                                                      last_id_field=False)
        return data_list, last_id

    @classmethod
    def get_basics_list_by_page(cls, page, count, **kwargs):
        """
        后台列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages, counts = cls.find_list_by_page(spec,
                                                         page,
                                                         count,
                                                         sort=[("create_at", -1), ],
                                                         fields=kwargs.get("fields", None))
        return data_list, pages, counts

    @classmethod
    def create_basics(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify_basics(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete_basics(cls, basics_id, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        return cls.update({"_id": basics_id}, {"state": -1})







# -*- coding:utf-8 -*-
import os
from setuptools import setup, find_packages

name = "lcyframe"
version = "3.7.8"

setup(name=name,
      version=version,
      description="A Fast ApiServer Frame for python3",
      long_description="",
      classifiers=[],
      keywords=name,
      author="lcylln",
      author_email="123220663@qq.com",
      url='',
      license='',
      platforms='any',
      packages=find_packages(),
      zip_safe=False,
      include_package_data=True,
      entry_points={
                'console_scripts': [
                    '%s = %s.__main__:main' % (name, name),
                ]
            },
      install_requires=[
          'tornado==5.0',
          'tornado-redis',
          'sphinx==1.6.3',
          'sphinx-rtd-theme==0.2.4',
          'mistune==0.8.4',     # 与m2r配合，生成文档 md
          'm2r>=0.1.12',
          'ruamel.yaml>=0.12.5',
          'normalize',
          'pyjwt>=1.5.3',
          'pyssdb>=0.4.2',
          'redis>=2.10.6',
          'celery>=4.4.7',
          'pynsq>=0.8.1',
          'mongoengine>=0.24.2',
          'pymongo>=4.0.2',
          'pymysql>=1.0.2',
          'DBUtils>=2.0.1',
          'pyaml>=17.10.0',
          'falcon>=1.3.0',
          'paho-mqtt>=1.3.1',
          'jinja2>=3.0.1',
          'xlrd>=2.0.1',
          'rsa>=4.7.2',
          'pycryptodome>=3.14.0',
          'envyaml>=1.10.211231'
      ],

)





import os, sys
import re, requests
import logging
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)

import os, sys
import re, requests
import logging
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)

from context import InitContext
from lcyframe.libs.singleton import MongoCon
from lcyframe.libs.cprint import cprint
from lcyframe.libs.JWT import JwtToken
from utils import helper

HEARDERS = {}
token = ""
config = InitContext.get_context()
# db = MongoCon().get_database(**config["mongo_config"])

def gen_token(user_id):
    return JwtToken(config["token_config"]["secret"]).encode({"user_id": user_id})

def login(account=None, password=None):
    headers = {"token": "******"}
    return send(
        "post",
        "/login",
        {
            # "account": account or "'yourname' or '1=1';",    # 注入攻击测试
            "account": account or "yourname",
            "password": password or "123456!",
        },
        headers=headers
    )

def send(methed, url, params, **kwargs):

    if "/" not in url:
        url = "/" + url

    global HEARDERS, token
    if "headers" not in kwargs:
        kwargs["headers"] = HEARDERS
    else:
        HEARDERS.update(kwargs["headers"])
        kwargs["headers"] = HEARDERS

    if "token" not in kwargs["headers"]:
        response = login()
        if response.get("code"):
            raise Exception(response["msg"])
        user = response
        token = user["token"]
        user_id = user["user_id"]
        HEARDERS["user_id"] = user_id
        HEARDERS["token"] = token
        kwargs["headers"]["token"] = token
        kwargs["headers"]["user_id"] = user["user_id"]

    for k, v in kwargs["headers"].items():
        if isinstance(v, int):
            kwargs["headers"][k] = str(v)

    _url = config["wsgi"]["host"] #

    if "http" not in config["wsgi"]["host"]:
        _url = "http://" + _url + ":" + str(config["wsgi"]["port"])
    elif "127.0.0.1" in _url:
        _url = config["wsgi"]["host"] + ":" + str(config["wsgi"]["port"])
    _url += url

    if methed == "delete":
        kwargs["data"] = params
        data = eval("requests.%s" % methed)(_url, **kwargs)
    else:
        data = eval("requests.%s" % methed)(_url, params, **kwargs)

    response = data.json()
    if response["code"] == 0:
        data = response["data"]
    else:
        data = response
    cprint(data)
    return data

def get(url, params, **kwargs):
    send("get", url, params, **kwargs)

def post(url, params, **kwargs):
    send("post", url, params, **kwargs)

def put(url, params, **kwargs):
    send("put", url, params, **kwargs)

def delete(url, params, **kwargs):
    send("delete", url, params, **kwargs)



# coding=u8

'''
库名：cprint
函数名：cprint()
功能：格式化输出list/set/tuple/dict等复杂类型对象，支持中文正常显示。
 '''

import sys

def __create_spaces(lvl):
    spaces = ''
    tab = '    '
    for i in range(lvl-1):
        spaces = spaces + tab
    return spaces

def __print(obj,end='\n'):
    if isinstance(obj,(str, int, float)):
        sys.stdout.write(str(obj)+end)
    elif obj is None:
        sys.stdout.write("None" + end)
    else:
        print(obj)
        sys.stdout.write(end)

def cprint(obj,lvl=0,indict=False,islast=True):
    lvl = lvl + 1
    spaces = __create_spaces(lvl)
    index = 0

    if isinstance(obj,list):
        if not indict:
            __print(spaces,end='')
        __print('[')
        for item in obj:
            if index+1 < len(obj):
                cprint(item,lvl,islast=False)
            else:
                cprint(item,lvl)
            index = index + 1
        __print(spaces + ']',end='')
    elif isinstance(obj,tuple):
        if not indict:
            __print(spaces,end='')
        __print('(')
        for item in obj:
            if index+1 < len(obj):
                cprint(item,lvl,islast=False)
            else:
                cprint(item,lvl)
            index = index + 1
        __print(spaces + ')',end='')
    elif isinstance(obj,set):
        if not indict:
            __print(spaces,end='')
        __print('set(')
        for item in obj:
            if index+1 < len(obj):
                cprint(item,lvl,islast=False)
            else:
                cprint(item,lvl)
            index = index + 1
        __print(spaces + ')',end='')
    elif isinstance(obj,dict):
        __print(spaces + '{')
        for key in obj:
            __print(__create_spaces(lvl+1) + '"' + str(key) + '"' + ': ',end='')
            if index+1 < len(obj):
                cprint(obj[key],lvl,indict=True,islast=False)
            else:
                cprint(obj[key],lvl,indict=True)
            index = index + 1
        __print(spaces + '}',end='')
    elif obj is None:
        __print(obj, end='')
    else:
        if isinstance(obj, str):
            obj = '"' + obj + '"'
        if not indict:
            __print(spaces,end='')
        __print(obj,end='')

    if islast:
        __print('')
    else:
        __print(',')



if __name__ == '__main__':
    obj1 = {
        'data':[
            {
            'data_id':'1',
            'content':'西红柿'
            },
            {
            'data_id':'2',
            'content':'胡萝卜'
            }
        ],
        'name':'蔬菜',
        'id':1,
    }
    print(obj1)
    print('\n')
    import pprint
    pprint.pprint(obj1)
    print('\n')
    cprint(obj1)



import os

import zipstream


def file_iterator(file_path, chunk_size=512):
    """
    文件生成器,防止文件过大，导致内存溢出
    :param file_path: 文件绝对路径
    :param chunk_size: 块大小
    :return: 生成器
    """
    with open(file_path, mode='rb') as f:
        while True:
            c = f.read(chunk_size)
            if c:
                yield c
            else:
                break


class ZipUtilities(object):
    """
    打包文件成zip格式的工具类
    使用方式
    # >>> utilities = ZipUtilities()
    # >>> for file_obj in file_objs:
    # >>>    tmp_dl_path = os.path.join(path_to, filename)
    # >>> utilities.to_zip(tmp_dl_path, filename)
    # >>> utilities.close()
    # >>> response = StreamingHttpResponse(utilities.zip_file, content_type='application/zip')
    # >>> response['Content-Disposition'] = 'attachment;filename="{0}"'.format("下载.zip")
    """
    zip_file = None

    def __init__(self):
        self.zip_file = zipstream.ZipFile(mode='w', compression=zipstream.ZIP_DEFLATED)

    def to_zip(self, file, name):
        if os.path.isfile(file):
            self.zip_file.write(file, arcname=os.path.basename(file))
        else:
            self.add_folder_to_zip(file, name)

    def add_folder_to_zip(self, folder, name):
        for file in os.listdir(folder):
            full_path = os.path.join(folder, file)
            if os.path.isfile(full_path):
                self.zip_file.write(full_path, arcname=os.path.join(name, os.path.basename(full_path)))
            elif os.path.isdir(full_path):
                self.add_folder_to_zip(full_path, os.path.join(name, os.path.basename(full_path)))

    def close(self):
        if self.zip_file:
            self.zip_file.close()


import os, sys, django, json
project_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(base_dir)


keys = []
names = []
class RBACPermission(object):
    PERMISSION_TABLE = "permission_group"
    PERMISSION_CLASS_TABLE = "permission_class"

    CURRENT_PERMISSION_GROUP = None
    CURRENT_PERMISSION_CLASS = None
    __VERSION__ = 0
    MAX_NUMS = 63
    IS_DEBUG = False
    default_group_config = None
    default_class_config = None
    mongo = MongoCon._database

    @classmethod
    def initial(cls):
        if cls.mongo is None:
            cls.mongo = MongoCon._database
            if cls.mongo is None:
                raise Exception("you must get a mongo isinstance!")

        if cls.CURRENT_PERMISSION_GROUP is None:
            cls.__load_permission_group()
        if cls.CURRENT_PERMISSION_CLASS is None:
            cls.__load_permission_class()

        return cls

    def has_permission(request):
        '''
        根据用户权限条目判断当前用户是否有权限请求
        :param request:
        :param permission_dict:
        :return:
        '''
        msg = None
        has_perm = False

        resolve_url_obj = resolve(request.path)
        current_url_name = resolve_url_obj.url_name
        current_method = request.method.lower()
        groups_permission = get_groups_mapping()
        if request.user.group_id in groups_permission:
            permission_dict = groups_permission[request.user.group_id]
            if permission_dict["state"]:         # 角色禁用，视为无权限
                key = current_url_name + ":" + current_method
                val = permission_dict["permissions"].get(key)
                if val:               # 1有权限，0无权限, None
                    has_perm = True
                    permission_mapping = get_permissions_mapping()
                    if key in permission_mapping:
                        permission = permission_mapping[key]
                        hook_func = permission["hook_func"]
                        if hook_func and hasattr(permission_hook, hook_func):
                            try:
                                results = getattr(permission_hook, hook_func)(request)
                                if isinstance(results, tuple) and len(results) == 2:
                                    flag, msg = results
                                else:
                                    flag, msg = results, ""
                            except Exception as e:
                                print(str(e))
                            else:
                                if flag:
                                    has_perm = True

        return has_perm, msg


    def init_permission(current_user, request):
        '''
        初始化所有权限条目
        :param current_user:
        :param request:
        :return:
        '''
        return None   # 不需要初始化
        # 获取当前用户的所有权限条目
        permission_item_queryset = get_permissions()
        permission_dict = {}
        for item in permission_item_queryset:
            permission_dict[item.code_name] = [
                item.url_name,
                item.permission_method,
                item.permission_args,
                item.permission_kwargs,
                item.permission_hook_func,
                item.menu_id,
            ]
        request.session[settings.PERMISSION_SESSION_KEY] = permission_dict
        logger.debug("%s", permission_dict)


    def check_permission(func):
        @wraps(func)
        def inner(*args, **kwargs):
            if not has_permission(*args, **kwargs):
                return JsonResponse({
                    'code': 403,
                    'msg': '没有权限'
                })
            return func(*args, **kwargs)

        return inner

    def rmcache(func):
        @wraps(func)
        def remove(*args, **kwargs):
            redis.delete("groups:%s" % 1)
            redis.delete("permissions:%s" % 1)
            redis.delete("company_groups:%s" % 1)
            redis.delete("groups:%s" % 0)
            redis.delete("permissions:%s" % 0)
            redis.delete("company_groups:%s" % 0)
            return func(*args, **kwargs)

        return remove

    def create_groups(**group):
        """
        创建角色
        name = models.CharField(max_length=128)
        state = models.IntegerField(default=1)
        desc = models.CharField(max_length=128, default="", verbose_name='描述')
        permissions = models.JSONField(default=dict)
        content_type = models.IntegerField(default=1)
        :return:
        """
        if Groups.objects.filter(name=group["name"]):
            return False
        else:
            return Groups.objects.create(**group)

    @rmcache
    def update_user_group(*uids, group_id=None):
        """
        修改用户的角色
        :param uids:
        :param kwargs: 新角色的id或0
        :return:
        """
        try:
            user = User.objects.get(id=uids[0])
            company = Company.objects.get(id=user.company_id)
        except Exception as e:
            return
        if not CompanyGroups.objects.filter(company_id=company.id, group_id=group_id):
            return
        User.objects.filter(id__in=uids).update(group_id=group_id)

    @rmcache
    def update_company_group(company_id, group_ids):
        """
        修改、新增商户的角色
        :param old_group:
        :param new_group:
        :return:
        """
        group_ids = [group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids
        for group_id in group_ids:
            CompanyGroups.objects.update_or_create(company_id=company_id, group_id=group_id)

    @rmcache
    def remove_company_group(company_id, group_ids):
        """
        移除商户的某个角色：所有已分配该角色的用户，将被重置为无角色
        :param company_id:
        :param group_id:
        :return:
        """
        group_ids = [group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids
        for group_id in group_ids:
            User.objects.filter(company_id=company_id, group_id=group_id).update(group_id=0)
            try:
                data = CompanyGroups.objects.get(company_id=company_id, group_id=group_id)
                data.delete()
            except Exception as e:
                pass

    @rmcache
    def update_group_permission(group_id, key, method, val):
        """
        勾选、取消权限
        :param group_id:
        :param val:
        :return:
        """
        group = Groups.objects.get(id=group_id)
        if group:
            permissions = group.permissions
            for method in method.split(","):
                if val:
                    permissions[key+":"+method] = val
                else:
                    permissions.pop(key+":"+method, "")
            group.save()


    def get_permissions_list(content_type=1):
        """
        权限条目导出为json,该数据结构返回给前端
        :return:
        """
        permissions = redis.get("permissions:%s" % content_type)
        if permissions:
            permissions = json.loads(permissions)
        else:
            permissions = []     # 顶级
            all_mapping = {}     # 所有数据的key：value
            datas = list(Permissions.objects.filter(state=1, content_type=content_type))   # 按id正序读取
            for p in datas:
                p = model_to_dict(p)
                if not p["parent_id"]:
                    permissions.append(p)
                all_mapping[p["id"]] = p
                if p["parent_id"]:
                    all_mapping[p["parent_id"]].setdefault("children", [])
                    all_mapping[p["parent_id"]]["children"].append(p)

            if permissions:
                redis.set("permissions:%s" % content_type, json.dumps(permissions))
        return permissions

    def get_permissions_mapping(content_type=1):
        """
        key与权限映射: 若用户某个权限值为1，则根据权限的基础信息，如hook_func执行对应操作
        :return:
        {
            key:method: 1 or 0, # methods含多个方法时，平铺为多个key和单方法，例如key:method
            key:get: 1,         # 键：权限key:方法(单个方法)
            key:post: 1,        # 键：权限key:方法(单个方法)
        }
        """
        permissions = get_permissions_list(content_type)
        permissions_mapping = {}
        def collect_api_permission(item, mapping):
            if item.get("children"):
                childrens = item.pop("children")
                for children in childrens:
                    collect_api_permission(children, mapping)
            else:
                # 权限禁用后，不做校验，视为有权限
                if item["permission_class"] == "api" and item["state"]:
                    for method in item["method"].split(","):
                        mapping[item["key"] + ":" + method] = item
                else:
                    return mapping

        for p in permissions:
            collect_api_permission(p, permissions_mapping)
        return permissions_mapping

    def get_groups_list(content_type=1):
        """
        返回角色（含权限），返回给前端
        :return:
        [
            {
                "name": "角色名称",
                "group_class":
                "state": 1,                           # 1可以 0 禁用 默认1
                "permissions": {
                    "control2-xxx-api:get": 1,
                    "control2-xxx-api2:get,post,put,delete": 1,  # 组合方法
                    "monitoring-tasks-api:post": 1
                }
            },
            ...
        ]
        """
        groups = redis.get("groups:%s" % content_type)
        if groups:
            groups = json.loads(groups)
        else:
            groups = []
            for g in Groups.objects.filter(state=1, content_type=content_type):
                groups.append(model_to_dict(g))
            if groups:
                redis.set("groups:%s" % content_type, json.dumps(groups))
        return groups

    def get_groups_mapping(content_type=1):
        """
        获取角色-权限映射
        :return:
        {
            1: {
                "...": **groups,                  # 角色基础信息
                "permissions": {
                    groups.key:get: 1,         # 键：权限key:方法(单个方法)
                    groups.key:post: 1,        # 键：权限key:方法(单个方法)
                    ...
                }
            }
        }
        """
        groups = {g["id"]: g for g in get_groups_list(content_type)}
        for id, group in groups.items():
            temp = {}
            permissions = group.pop("permissions")
            # get,post 多个方法平铺
            for keys, val in permissions.items():
                if not val:
                    continue  # 0代表没有权限，跳过
                key, methods = keys.split(":")
                if "," in methods:
                    for method in methods.split(","):
                        temp[key + ":" + method] = val
                else:
                    temp[keys] = val
            group["permissions"] = temp
        return groups

    def get_company_groups(content_type=1):
        """
        商户对应的角色
        :param content_type:
        :return:
        """
        company_groups = {}
        rmcache_datas = redis.get("company_groups:%s" % content_type)
        if rmcache_datas:
            company_groups = json.loads(rmcache_datas)
        else:
            datas = CompanyGroups.objects.all()
            for item in datas:
                if item.company_id not in company_groups:
                    company_groups[item.company_id] = []
                company_groups[item.company_id].append(item.group_id)
            if company_groups:
                redis.set("company_groups:%s" % content_type, json.dumps(company_groups))

        return company_groups

if __name__ == '__main__':
    # create_groups(**{"name": "新角色"})
    # update_company_group(2, [5,8])
    # update_user_group(58, group_id=5)
    # remove_company_group(2, 5)
    # update_group_permission(5, "user_detail", "get", 1)

    print(get_permissions_list(1))
    print(get_permissions_mapping(1))
    print(get_groups_list(1))
    print(get_company_groups(1))
    pass


# -*- coding:utf-8 -*-
import os
import sys
import io
import re
import glob
import logging
from envyaml import EnvYAML
from jinja2 import Environment, FileSystemLoader, Template
from ruamel.yaml import round_trip_dump, round_trip_load, load, dump_all
from importlib import import_module
import inspect

resource_mp = {}

def impmodule(base, model_dir=None):
    """
    import model class obj global
    warn: the class obj cant not start whit "__"
    :return:
    """
    ROOT = os.environ.app_config["ROOT"]
    model = type("model", (object,), {})
    for m in [x for x in os.listdir(model_dir) if re.findall('[A-Za-z]\w+_model\.py$', x)]:
        m = m[:-3]
        if m == "__init__":
            continue

        model_name = model_dir.replace(ROOT, "").lstrip("/").replace("/", ".") + "." + m.rstrip(".py")
        # for attr_name, value in import_module(model_name).__dict__.items():
        for attr_name, value in inspect.getmembers(import_module(model_name), inspect.isclass):
            if attr_name.startswith("__"):
                continue

            if not hasattr(model, attr_name):
                setattr(model, attr_name, value)
                logging.debug("model [%s] register success!" % attr_name)

    setattr(base, "model", model)

def load_jinja2(name, jinja2_path=""):
    """
    dir must abd path
    :param dir:
    :return:
    """
    if jinja2_path and os.path.exists(jinja2_path):
        jinja_env = Environment(loader=FileSystemLoader(jinja2_path))
        template = jinja_env.get_template('%s.py.j2' % name)
    else:
        jinja2_path = os.path.join(os.path.dirname(__file__), "template")
        if os.path.exists(os.path.join(jinja2_path, name) + ".py.j2"):
            jinja_env = Environment(loader=FileSystemLoader(jinja2_path))
            template = jinja_env.get_template('%s.py.j2' % name)
        else:
            sys.path.append(jinja2_path)
            j2 = import_module("template")
            str = getattr(j2, "%s_j2" % name).decode("u8")
            template = Template(str)

    return template


def snake_to_camel(snake_str):
    return _convert_to_camel(snake_str, '_')


def snake_to_cap(snake_str):
    return _convert_to_camel(snake_str, '_', True)


def _convert_to_camel(snake_cased_str, separator, first_cap=False):
    components = snake_cased_str.split(separator)
    preffix = ""
    suffix = ""
    if components[0] == "":
        components = components[1:]
        preffix = separator
    if components[-1] == "":
        components = components[:-1]
        suffix = separator
    if len(components) > 1:
        camel_cased_str = components[0].title() if first_cap else components[0].lower()
        for x in components[1:]:
            if x.isupper() or x.istitle():
                camel_cased_str += x
            else:
                camel_cased_str += x.title()
    else:
        camel_cased_str = components[0].title()
    return preffix + camel_cased_str + suffix


def extract_dict(some_dict, some_keys):
    return dict([(k, some_dict[k]) for k in some_keys if k in some_dict])


def reduce_result_keys(result):
    if not result:
        return
    record = {}
    for k, v in result.items():
        record[k] = extract_dict(v, ('name', 'type', 'required', 'schema', 'rename'))
    return record


def _filter_params(pkg_dir, schema, ignore_type=["bool", "BOOL"]):
    for f, item in schema["method"].items():
        parameters = item.get("parameters") or []
        if type(parameters) != list:
            raise Exception("parameters must is List. in %s:%s" % (pkg_dir, item["summary"]))
        item["parameters"] = parameters

        responses = item.get("responses") or {}
        if responses is None:
            raise Exception("parameters can not is None. in %s:%s" % (pkg_dir, item["summary"]))
        item["responses"] = responses

        for params in item["parameters"] or []:
            params_name = params["name"]
            for k, v in params.items():
                if k == "type" and v.lower() in ignore_type:
                    raise Exception("params '%s' can not is bool type in %s:%s" % (params_name, pkg_dir, item["summary"]))

            # if params["type"] == "json" and not params["required"] and params.get("default") is None:
            #     raise Exception(
            #         "params '%s' is json type. required true or give default in %s" % (params_name, pkg_dir))

        for k, v in item["responses"].items():
            if v is None:
                raise Exception("responses key '%s' value can not is 'None' or empty in %s:%s" % (k, pkg_dir, item["summary"]))


def dump_yaml_file(data, path, round_tripping=False):
    with io.open(path, 'w', encoding='utf-8') as writer:
        if round_tripping:
            round_trip_dump(data, writer, allow_unicode=True, )
        else:
            dump_all([data], writer, allow_unicode=True)


def load_yaml_file(path, round_tripping=False):
    with io.open(path, 'r', encoding='utf-8') as reader:
        if round_tripping:
            data = round_trip_load(reader)
        else:
            # data = load(reader)
            import warnings, ruamel.yaml
            # warnings.simplefilter('ignore', ruamel.yaml.error.UnsafeLoaderWarning)
            data = load(reader, Loader=ruamel.yaml.Loader)
    return data


def load_confog(path, round_tripping=False):
    return load_yaml_file(path, round_tripping)

def load_confog_v2(path):
    return EnvYAML(path)

def generate_basepy(path, jinja2_path_dir):
    template = load_jinja2("base", jinja2_path_dir)

    s = template.render(resources={})

    p = path
    with io.open(p, 'w', encoding='utf-8') as w:
        w.write(s)

    logging.debug("base.py", ' .. created')


def load_api_schema(api_schema_dir):
    api_schema_mp = {}

    for pkg_dir in glob.glob('%s/*' % api_schema_dir):
        if os.path.isdir(pkg_dir):
            api_schema_mp.update(load_api_schema(pkg_dir))
        else:
            # for mod_file in glob.glob('%s/*.yml' % pkg_dir):
            resources = []

            if os.path.basename(pkg_dir).count(".") > 1:
                raise Exception("%s file name exists multiple '.'" % (pkg_dir))

            mod_file_name = os.path.basename(pkg_dir).split(".")[0]
            model = ""
            resources.extend(load_yaml_file(pkg_dir, False))
            for res in resources:

                _filter_params(pkg_dir, res)

                api = res["apis"]
                if api in api_schema_mp:
                    raise Exception("%s has multiple Api named '%s'; error" % (mod_file_name, api))

                if not model:
                    model = res.get("model", "")
                api_schema_mp[api] = {
                    "name": mod_file_name,
                    "model": model,
                    "api": res["apis"],
                    "api_title": res.get("name", ""),
                    "api_name": res["handler"],
                    "description": res.get("description", "") or "",
                    "method": res["method"],
                }
                logging.debug('params [%s] load success!' % api)

    return api_schema_mp


def process_resource_context(model_path, resource_def):
    # handler
    resource_def['handler_name'] = resource_def["api_name"][0].upper() + resource_def["api_name"][1:] + "Handler"

    # model
    resource_def["model_dir"] = os.path.basename(model_path)
    resource_def['model_name'] = resource_def["name"][0].upper() + resource_def["name"][1:] + "Model"

    # schema
    resource_def['schema_name'] = resource_def["name"][0].upper() + resource_def["name"][1:] + "Schema"

    return resource_def


def load_jinja2_handler(model_path, resource_text, jinja2_path=None):
    template = load_jinja2("handler", jinja2_path)

    resources = []
    for resource_def in resource_text:
        resource_context = process_resource_context(model_path, resource_def)
        resources.append(resource_context)

    s = template.render(resources=resources)
    return s


def load_jinja2_model(model_path, resource_text, jinja2_path=None):
    template = load_jinja2("model", jinja2_path)

    resources = []
    for resource_def in resource_text:
        resource_context = process_resource_context(model_path, resource_def)
        resources.append(resource_context)

    s = template.render(resources=resources)
    return s


def load_jinja2_model_schema(model_path, resource_text, jinja2_path=None):
    template = load_jinja2("schema", jinja2_path)

    resources = []
    for resource_def in resource_text:
        resource_context = process_resource_context(model_path, resource_def)
        resources.append(resource_context)

    s = template.render(resources=resources)
    return s

def load_jinja2_testscript(testscript_path, resource_text, jinja2_path=None):
    template = load_jinja2("restfull", jinja2_path)

    resources = []
    for resource_def in resource_text:
        resource_context = process_resource_context(testscript_path, resource_def)
        resources.append(resource_context)

    s = template.render(resources=resources)
    return s

def generate_resource_handler(handler_path, model_path, api_schema_mp, jinja2_path_dir):
    resource_mp = {}
    for api, item in api_schema_mp.items():
        resource_mp.setdefault(item["name"], [])
        resource_mp[item["name"]].append(item)

    for mod_name, item in resource_mp.items():
        p = os.path.join(handler_path, mod_name + '_handler.py')
        if os.path.exists(p):
            logging.debug("handler [%s_handler.py] exists.. skip" % mod_name)
            continue

        mod_py_source = load_jinja2_handler(model_path, item, jinja2_path_dir)

        with io.open(p, 'w', encoding='utf-8') as w:
            w.write(mod_py_source)

        logging.debug("handler [" + mod_name + "_handler.py].. created")


def generate_resource_model(model_path, api_schema_mp, jinja2_path_dir):
    resource_mp = {}
    for api, item in api_schema_mp.items():
        resource_mp.setdefault(item["name"], [])
        resource_mp[item["name"]].append(item)

    for mod_name, item in resource_mp.items():
        p = os.path.join(model_path, mod_name + '_model.py')
        if os.path.exists(p):
            logging.debug("model [%s_model.py] exists.. skip" % mod_name)
            continue

        mod_py_source = load_jinja2_model(model_path, item, jinja2_path_dir)

        with io.open(p, 'w', encoding='utf-8') as w:
            w.write(mod_py_source)

        logging.debug("model [" + mod_name + "_model.py].. created")


def generate_resource_model_schema(schema_path, api_schema_mp, jinja2_path_dir):
    resource_mp = {}
    for api, item in api_schema_mp.items():
        resource_mp.setdefault(item["name"], [])
        resource_mp[item["name"]].append(item)

    for mod_name, item in resource_mp.items():
        p = os.path.join(schema_path, mod_name + '_schema.py')
        if os.path.exists(p):
            logging.debug("schema [%s_schema.py] exists.. skip" % mod_name)
            continue

        mod_py_source = load_jinja2_model_schema(schema_path, item, jinja2_path_dir)

        with io.open(p, 'w', encoding='utf-8') as w:
            w.write(mod_py_source)

        logging.debug("schema [%s_schema.py] .. created success" % mod_name)

def generate_resource_testscript(testscript_path, api_schema_mp, jinja2_path_dir):
    resource_mp = {}
    for api, item in api_schema_mp.items():
        resource_mp.setdefault(item["name"], [])
        resource_mp[item["name"]].append(item)

    for mod_name, item in resource_mp.items():
        p = os.path.join(testscript_path, mod_name + '.py')
        if os.path.exists(p):
            logging.debug("restfull script [%s.py] exists.. skip" % mod_name)
            continue

        mod_py_source = load_jinja2_testscript(testscript_path, item, jinja2_path_dir)

        with io.open(p, 'w', encoding='utf-8') as w:
            w.write(mod_py_source)

        logging.debug("restfull script [%s.py] .. created success" % mod_name)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.permission_schema import GroupsSchema, CompanyGroupsSchema, PermissionsSchema
from utils.rbac import permission_hook


class RBACPermissionModel(BaseModel, PermissionsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "parent_id" in doc and doc["parent_id"] == None:
            doc["parent_id"] = 0
        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def has_permission(cls, handler):
        '''
        根据用户权限条目判断当前用户是否有权限请求
        :param request:
        :param permission_dict:
        :return:
        '''
        msg = None
        has_perm = False

        current_path = handler.request.path
        current_key = handler.router_name
        current_method = handler.request.method.lower()

        # # 企业账号免校验，拥有所有权限
        # if request.user.is_superuser:
        #     return True, msg

        # 是否启用权限系统
        if cls.is_enable(0) != True:
            return True, msg

        key = current_key + ":" + current_method
        permission_mapping = cls.get_permissions_mapping(0)

        # TODO key不存在代表权限不存在，视为有权限.不存在的权限通常为基础接口或必须允许访问的接口
        # if key not in groups["permissions"] and key not in permission_mapping:
        #     has_perm = True
        #     break

        # 权限不存在，视为无权限
        if key not in permission_mapping:
            return False, msg

        permission = permission_mapping[key]
        # 权限已禁用，视为有权限,不执行hook_func
        if permission["state"] == 0:
            return True, msg

        groups_permission = cls.model.GroupsModel.get_groups_mapping(0)
        # 支持用户有多个角色，只要其中一个角色有权限，即代表有权限
        for group_id in str(handler.member["group_id"]).split(','):
            if str(group_id) not in groups_permission:
                continue

            groups = groups_permission[str(group_id)]
            # 角色禁用，视为无权限,即使该角色拥有所有权限
            if groups["state"]:
                # 未勾选
                if key not in groups["permissions"]:
                    continue

                val = groups["permissions"][key]
                # 已勾选, 执行hook_func
                if val:
                    has_perm = True
                    hook_func = permission["hook_func"]
                    if hook_func and hasattr(permission_hook, hook_func):
                        try:
                            results = getattr(permission_hook, hook_func)(handler.request)
                            if isinstance(results, tuple) and len(results) == 2:
                                flag, msg = results
                            else:
                                flag, msg = results, ""
                        except Exception as e:
                            print(str(e))
                        else:
                            if flag:
                                has_perm = True

                    if has_perm:
                        break
                    else:
                        continue

        return has_perm, msg

    @classmethod
    def create_permission(cls, key, name, permission_class, content_type, **kwargs):
        """
        新增权限条目
        :param id:
        :param kwargs:
        :return:
        """

        kwargs["name"] = name
        kwargs["key"] = key
        kwargs["permission_class"] = permission_class
        kwargs["content_type"] = int(content_type)
        kwargs.setdefault("state", 1)

        cls.rmcache(content_type)
        return cls.create_data(**kwargs)

    @classmethod
    def update_permission(cls, permission_id, content_type, **kwargs):
        """
        单条更新权限信息: 名称 描述 状态（启用停用） 钩子 父级
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        key = kwargs.pop("key", "")  # 暂时不允许修改
        # kwargs.pop("method", "")
        permission_class = kwargs.pop("permission_class", "")  # 暂时不允许修改
        if kwargs:
            permission = cls.get_data_by_kv(id=permission_id)
            cls.update_data(permission_id, **kwargs)

            # 若修改方法，则已勾选该权限的角色，需要移除该权限
            if "method" in kwargs or permission_class in kwargs:
                groups = cls.model.GroupsModel.get_groups_mapping(content_type)
                for gid, group in groups.items():
                    is_modify = False
                    for method in permission["method"].split(","):
                        key = permission["key"] + ":" + method
                        if key in group["permissions"]:
                            val = group["permissions"].pop(key, 0)
                            # 新的method置换旧的method
                            group["permissions"][permission["key"] + ":" + kwargs["method"]] = val
                            is_modify = True
                    if is_modify:
                        cls.model.GroupsModel.update_group_permissions(int(gid), group["permissions"])

            cls.rmcache(content_type)

    @classmethod
    def remove_permission(cls, permission_id, content_type):
        """
        删除权限条目:物理删除
        :param id:
        :param kwargs:
        :return:
        """
        cls.rmcache(content_type)
        return cls.mysql.execute(f"delete from {cls.collection} where id=%s", [permission_id])

    @classmethod
    def get_permissions_mapping(cls, content_type=1):
        """
        权限的key:method映射
        :return:
        {
            key:method: permission,      # methods含多个方法时，平铺为多个key和单方法，例如key:method
            key:get: permission,         # 键：权限key:方法(单个方法)
            key:post: permission,        # 键：权限key:方法(单个方法)
        }
        """
        permissions = cls.get_permissions_list(content_type)
        permissions_mapping = {}

        def collect_api_permission(item, mapping):
            if item.get("children"):
                childrens = item.pop("children")
                for children in childrens:
                    collect_api_permission(children, mapping)
            else:
                # 权限禁用后\不存在，视为有权访问；没勾选视为无权限
                if item["permission_class"] == "api":
                    for method in item["method"].split(","):
                        mapping[item["key"] + ":" + method] = item
                else:
                    return mapping

        for p in permissions:
            collect_api_permission(p, permissions_mapping)
        return permissions_mapping

    @classmethod
    def get_permissions_list(cls, content_type=1):
        """
        前端
        权限条目
        导出为json,该数据结构返回给前端
        :return:
        [
            {permission1},      # {"name": "短信验证码","key":"smscode", "children": [],...}
            {permission2},
            {permission3},
        ]
        """
        permissions = cls.redis.get("permissions:%s" % content_type)
        if permissions:
            permissions = cls.utils.to_python(permissions)
        else:
            permissions = []  # 顶级
            all_mapping = {}  # 所有数据的key：value
            datas = list(cls.get_datas_by_spec(sql_and=dict(content_type=content_type)))  # 按id正序读取
            for p in datas:
                all_mapping[p["id"]] = p

                if not p["parent_id"]:
                    p["parent_id"] = 0
                    permissions.append(p)

            for id, p in all_mapping.items():
                if p["parent_id"]:
                    all_mapping[p["parent_id"]].setdefault("children", [])
                    all_mapping[p["parent_id"]]["children"].append(p)

            if permissions:
                cls.redis.set("permissions:%s" % content_type, cls.utils.to_json(permissions))
        return permissions

    @classmethod
    def enable_permission(cls, content_type=1):
        """
        启用权限系统
        :param content_type:
        :return:
        """
        cls.redis.set("permission_enable:%s" % content_type, 1)

    @classmethod
    def disable_permission(cls, content_type=1):
        """
        停用权限系统
        :param content_type:
        :return:
        """
        cls.redis.set("permission_enable:%s" % content_type, 0)

    @classmethod
    def is_enable(cls, content_type=1):
        """
        是否需要权限拦截
        """
        return cls.redis.get("permission_enable:%s" % content_type) == b'1'

    @classmethod
    def rmcache(cls, content_type=1):
        cls.remove_redis(content_type)
        cls.set_version(content_type)

    @classmethod
    def remove_redis(cls, content_type=1):
        cls.redis.delete("groups:%s" % content_type)
        cls.redis.delete("permissions:%s" % content_type)
        cls.redis.delete("company_groups:%s" % content_type)

    @classmethod
    def set_version(cls, content_type=1):
        """
        每一次修改权限，版本号为当前时间戳
        :param content_type:
        :return:
        """
        cls.redis.set("permission_version:%s" % content_type, cls.utils.now())

    @classmethod
    def get_version(cls, content_type=1):
        """
        获取当前版本号
        :param content_type:
        :return:
        """
        current_version = cls.redis.get("permission_version:%s" % content_type) or 0
        return int(current_version)

    @classmethod
    def has_update(cls, client_version, content_type=1):
        """
        客户端是否需要更新权限
        当客户端请求的header.permission_version<当前版本号时，
        设置响应header.permission_version=current_version
        客户端判断响应头里包含header.permission_version时，重新请求权限数据，并刷新客户端权限配置
        :param client_version:
        :param content_type:
        :return:
        """
        return cls.get_version(content_type) > client_version

    @classmethod
    def autoload_permission(cls, routes_hander_mp, permissions):
        """
        自动收集并创建权限条目
        该方法由进程启动时调用，请勿手动调用
        routers： self.application.handlers
        routes_hander_mp： key对应的hendler对象
        permissions: {"key": [get, post], "key1": [get, put]}
        """
        method_title = {"get": "查看", "post": "创建", "put": "修改", "delete": "删除"}
        datas = []
        for key, methods in permissions.items():
            handler = routes_hander_mp[key]
            class_docs = handler.__doc__
            for method in methods:
                method_docs = getattr(handler, method).__doc__
                # 优先选取方法描述
                if not method_docs:
                    # 其次选取类描述
                    if not class_docs:
                        name = method_title[method] + key
                        describes = "自动载入的权限"
                    else:
                        class_docs_list = class_docs.split("\n")
                        docs = class_docs_list[0] or class_docs_list[1]
                        name = method_title[method] + docs.replace(" ", "")[:10]
                        describes = class_docs.replace("\n", "").replace(" ", "")[:100]
                else:
                    method_docs_list = method_docs.split("\n")
                    docs = method_docs_list[0] or method_docs_list[1]
                    name = docs.replace(" ", "")[:10]
                    describes = method_docs.replace("\n", "").replace(" ", "")[:100]

                data = cls.create_data(**{
                    "key": key,
                    "name": name or method_title[method] + key,
                    "method": method,
                    "permission_class": "api",
                    "describes": describes,
                    "content_type": 0
                })
                datas.append(data)
        if datas:
            cls.rmcache(0)
        return datas

    @classmethod
    def import_permissions(cls, permissions, content_type=1, is_delete=False):
        """
        权限json导入db
        is_delete: 是否删除旧数据
        "content_type": 1 or 0              # 1 业务端使用，0系统管理端
        :return:
        """
        create_data = []
        if is_delete:
            cls.mysql.execute(f"DELETE FROM {cls.collection} where content_type=%s", [content_type])

        assert cls.check_field(permissions)

        def loop_create_item(item, parent=None):
            if cls.get_data_by_spec(sql_and={"key": item["key"], "method": item.get("method", "")}):
                return
            item.pop("id", "")  #
            parent = cls.create_data(
                name=item["name"],
                key=item["key"],
                method=item.get("method", ""),
                parent_id=parent["id"] if parent else None,
                permission_class=item.get("permission_class", ""),
                hook_func=item.get("hook_func", ""),
                state=item.get("state", 1),
                describes=item.get("describes", ""),
                content_type=content_type
            )
            # print("写入权限:%s" % item["name"])
            create_data.append(item)
            for children in item.get("children", []):
                loop_create_item(children, parent)

        for p in permissions:
            loop_create_item(p)

        return create_data

    @classmethod
    def export_permission(cls, content_type=1):
        """
        导出权限表为json
        """
        return cls.get_permissions_list(content_type)

    @classmethod
    def check_field(cls, permissions, KEYS=None, NAMES=None):
        """
        校验权限合法性
        :param item:
        :return:
        """
        if not KEYS or not NAMES:
            KEYS = []
            NAMES = []
        for p in permissions:
            if p.get("children", []):
                cls.check_field(p["children"], KEYS, NAMES)
            else:
                if not p["name"] or not p["key"] or not p["permission_class"]:
                    raise Exception("关键字都必填")
                # if p["permission_class"] == "tag" and not p["children"]:
                #     raise Exception("标签下尚未设置权限")
                if p["permission_class"] == "api" and not p["method"]:
                    raise Exception("API级别权限必须指定方法：get、post、put、delete")
                if p["permission_class"] == "logic" and not p["method"]:
                    raise Exception("暂不支持逻辑权限")

                if p["permission_class"] == "api":
                    unique_name = p["key"] + ":" + p["method"]
                else:
                    unique_name = p["key"]
                if unique_name not in KEYS:
                    KEYS.append(unique_name)
                else:
                    raise Exception("存在重复的key：%s" % p["key"])

                if p["name"] not in KEYS:
                    NAMES.append(p["name"])
                else:
                    raise Exception("存在重复的name：%s" % p["name"])

        return True


class GroupsModel(BaseModel, GroupsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "permissions" in doc:
            doc["permissions"] = cls.utils.to_python(doc["permissions"])
        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def get_members_count(cls, company_id, *groups):
        """
        属于某商户的角色的成员数
        """
        groups_mp = {}
        members = cls.model.AdminModel.get_datas_by_spec()
        for user in members:
            if company_id and company_id != user["company_id"]:
                continue
            for gid in user["group_id"].split(","):
                groups_mp.setdefault(gid, 0)
                groups_mp[gid] += 1

        for group in groups:
            group["counts"] = groups_mp.get(str(group["id"]), 0)

        return list(groups)

    @classmethod
    def get_members(cls, company_id, groups):
        """
        属于某商户且是该角色的成员
        """
        members = []
        for user in cls.model.AdminModel.get_datas_by_spec():
            if company_id and company_id != user["company_id"]:
                continue
            if groups["id"] in [int(gid) for gid in user["group_id"].split(",") if gid]:
                members.append(user)

        return members

    @classmethod
    def get_group_list_by_page(cls, page, count, **kwargs):
        """
        列表
        :return:
        :rtype:
        """

        datas, counts = cls.get_list_by_page(page, count, **kwargs)
        return datas, counts

    @classmethod
    def create_groups(cls, name, content_type, **group):
        """
        创建角色
        name = models.CharField(max_length=128)
        state = models.IntegerField(default=1)
        describes = models.CharField(max_length=128, default="", verbose_name='描述')
        permissions = models.JSONField(default=dict)
        content_type = models.IntegerField(default=1, verbose_name='1业务角色，2管理端角色')
        assign = models.IntegerField(default=1, verbose_name='是否默认分配给商户,1是0否')
        :return:
        """

        if content_type not in [0, 1]:
            raise Exception("参数错误")
        if cls.model.GroupsModel.get_data_by_spec(sql_and={"name": name, "content_type": content_type}):
            raise Exception("已存在同名角色")
        group["name"] = name
        group["content_type"] = content_type
        cls.model.RBACPermissionModel.rmcache(content_type)
        return cls.create_data(**group)

    @classmethod
    def update_group(cls, group_id, content_type, **kwargs):
        """
        更新角色信息: 名称 描述 状态 分配
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        kwargs.pop("permissions", "")
        if kwargs.get("name") and cls.model.GroupsModel.get_data_by_spec(
                sql_and={"name": kwargs["name"], "content_type": content_type}):
            raise Exception("已存在同名角色")
        if kwargs:
            cls.update_data(group_id, **kwargs)
            cls.model.RBACPermissionModel.rmcache(content_type)

    @classmethod
    def update_user_group(cls, *uids, group_id=None):
        """
        修改用户的角色
        :param uids:
        :param group_id: "gid1,gid2,gid3"
        :return:
        """
        try:
            user = cls.model.AdminModel.get_admin(id=uids[0])
            company = cls.model.CompanyModel.get_company(user["company_id"])
        except Exception as e:
            return
        if not cls.get_data_by_spec(sql_and={"company_id": company["id"], "group_id": group_id}):
            return

        for uid in uids:
            cls.model.AdminModel.update_data(uid, group_id=group_id)

    @classmethod
    def update_group_permission(cls, group_id, key, method, val):
        """
        单条勾选、取消权限
        :param group_id:
        :param key:
        :param method: get or get,post 多个方法用逗号隔开
        :return:
        """
        group = cls.get_data_by_kv(id=group_id)
        if group:
            permissions = group["permissions"]
            if val:
                permissions[key + ":" + method] = val
            else:
                permissions.pop(key + ":" + method, "")

            group["permissions"] = cls.utils.to_json(permissions)
            group.pop("id", "")
            cls.update_data(group_id, **group)
            cls.model.RBACPermissionModel.rmcache(group["content_type"])

    @classmethod
    def update_group_permissions(cls, group_id, permissions):
        """
        批量勾选、取消权限
        :param group_id:
        :param permissions:
            {
                "key:get": 1,
                "key:get,post": 1
            }
        :return:
        """
        save_permissions = {}
        group = cls.get_data_by_kv(id=group_id)
        if group:
            permissions_data = cls.model.RBACPermissionModel.get_datas_by_spec("key", "method", "permission_class",
                                                                               content_type=group["content_type"])
            permissions_mapping = {"%s:%s" % (item["key"], item["method"]): item for item in permissions_data if
                                   item["permission_class"] != "tag"}
            for key, val in permissions.items():
                # 过滤表中已被物理删除的权限
                if key not in permissions_mapping:
                    continue
                # 勾选了tag组，不保存进表
                if permissions_mapping[key]["permission_class"] == "tag":
                    continue
                save_permissions[key] = val
            cls.update_data(group_id, permissions=cls.utils.to_json(save_permissions))
            cls.model.RBACPermissionModel.rmcache(group["content_type"])

    @classmethod
    def get_groups_list(cls, content_type=1):
        """
        前端
        返回角色（含权限）
        :return:
        [
            {
                "id": "id",
                "name": "角色名称",
                # "group_class": "技术部",             # 预留，自定义标签
                "state": 1,                           # 1可以 0 禁用 默认1
                "content_type": 1 or 0,             # 1 业务端使用，0系统管理端
                "assign": 0,                        # 是否默认分配给商户 1是0否
                "permissions": {
                    "control2-xxx-api:get": 1,        # 1勾选，0未勾选
                    "control2-xxx-api2:get,post,put,delete": 1,  # 组合方法
                    "monitoring-tasks-api:post": 1
                }
            },
            ...
        ]
        """
        groups = cls.redis.get("groups:%s" % content_type)
        if groups:
            groups = cls.utils.to_python(groups)
        else:
            groups = []
            for g in cls.get_datas_by_spec(sql_and=dict(content_type=content_type)):
                groups.append(g)
            if groups:
                cls.redis.set("groups:%s" % content_type, cls.utils.to_json(groups))
        return groups

    @classmethod
    def get_groups_mapping(cls, content_type=1):
        """
        获取角色拥有的权限，多个方法组合为一个权限时，会得到多条k：v
        :return:
        {
            "1": {
                "...": **groups,                  # 角色基础信息
                "permissions": {
                    groups.key:get: 1,         # 键：权限key:方法(单个方法)
                    groups.key:post: 1,        # 键：权限key:方法(单个方法)
                    ...
                }
            }
        }
        """
        groups = {str(g["id"]): g for g in cls.get_groups_list(content_type)}
        permissions_mapping = cls.model.RBACPermissionModel.get_permissions_mapping(content_type)
        for id, group in groups.items():
            temp = {}
            permissions = group.pop("permissions")
            # get,post 多个方法平铺
            for keys, val in permissions.items():
                if not val:
                    continue  # 0代表没有权限，跳过
                key, methods = keys.split(":")
                for method in methods.split(","):
                    # 权限记录已经不存在了，跳过(手动删除权限表记录会出现)
                    if key + ":" + method not in permissions_mapping:
                        continue
                    else:
                        temp[key + ":" + method] = val
            group["permissions"] = temp
        return groups

    @classmethod
    def get_system_groups(cls):
        """
        获取系统角色
        :return: [1, 2, 3]
        """
        return cls.model.CompanyGroupsModel.get_company_groups(0)


class CompanyGroupsModel(BaseModel, CompanyGroupsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def update_company_group(cls, company_id, group_ids):
        """
        修改、新增商户的角色
        给商户指派角色
        :param old_group:
        :param new_group:
        :return:
        """
        if company_id == 1:
            raise Exception("禁止修改系统端角色")
            groups_mp = cls.model.GroupsModel.get_groups_mapping(0)
            old_group_id = cls.get_system_groups()
        else:
            groups_mp = cls.model.GroupsModel.get_groups_mapping(1)
            company_groups_mp = cls.get_company_groups(1)
            old_group_id = company_groups_mp.get(str(company_id), [])

        group_ids = set([group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids)
        diff_group_id = set(old_group_id) - (set(group_ids))
        group_ids.difference_update(set(old_group_id))
        for group_id in group_ids:
            if str(group_id) not in groups_mp:
                continue
            data = cls.get_data_by_spec(sql_and=dict(company_id=company_id, group_id=int(group_id)))
            if not data:
                cls.create_data(company_id=company_id, group_id=group_id)
            else:
                cls.update_data(data["id"], company_id=company_id, group_id=group_id)

        # 其他的角色，移除
        if diff_group_id:
            cls.remove_company_group(company_id, list(diff_group_id))
        cls.model.RBACPermissionModel.rmcache(1)

    @classmethod
    def remove_company_group(cls, company_id, group_ids):
        """
        移除商户的某个角色：所有已分配该角色的用户，将被解除角色绑定
        :param company_id:
        :param group_id:
        :return:
        """
        group_ids = [str(i) for i in ([group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids)]
        # 删除角色
        for group_id in group_ids:
            cls.mysql.execute(f"delete from {cls.collection} where company_id=%s and group_id=%s",
                              [company_id, int(group_id)])

        # 从该用户角色中剔除这些角色
        for user in cls.model.AdminModel.get_datas_by_spec(sql_and=dict(company_id=company_id)):
            user_group_ids = set(user["group_id"].split(","))
            group_ids = set(group_ids)
            user_group_ids.difference_update(group_ids)
            user["group_id"] = ",".join(list(user_group_ids))
            cls.model.AdminModel.update_data(user["id"], group_id=user["group_id"])

        cls.model.RBACPermissionModel.rmcache(1)

    @classmethod
    def get_system_groups(cls):
        """
        获取系统角色
        :return: [1, 2, 3]
        """
        return cls.get_company_groups(0)

    @classmethod
    def get_company_groups(cls, content_type=1):
        """
        商户对应的角色
        :param content_type:
        :return:["2": [1, 2, 3], "3": [4, 5, 6]]
        """
        company_groups = {}
        cache_datas = cls.redis.get("company_groups:%s" % content_type)
        if cache_datas:
            company_groups = cls.utils.to_python(cache_datas)
        else:
            if content_type == 1:
                datas = cls.get_datas_by_spec()
                for item in datas:
                    if str(item["company_id"]) not in company_groups:
                        company_groups[str(item["company_id"])] = []
                    company_groups[str(item["company_id"])].append(item["group_id"])
            else:
                company_groups = []
                for group in cls.model.GroupsModel.get_datas_by_spec(sql_and=dict(content_type=content_type)):
                    company_groups.append(group["id"])

            if company_groups:
                cls.redis.set("company_groups:%s" % content_type, cls.utils.to_json(company_groups))

        return company_groups